#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
=======================================================
plot DNS dynamo and rotating convection tau-ell diagram
=======================================================
Small script to plot a tau-ell regime diagram for geodynamo simulations
From Treatise on Geophysics 'Turbulence in the core' chapter
Nataf H-C. and N. Schaeffer, 2015.

240205 HCN : new version for shell *** remaining details to be considered
230412 HCN : a few adjustments for Guervilly. *** template to be modified 
230408 HCN : add QG diss + u_r tau-ell for Guervilly+ 3D simu (P_diss + template to be improved)
230405 HCN : correct computation of P_diss (sum instead of integral)
230327 HCN : add wavy lines for Rossby and Alfven
230321 HCN : divide P_diss by 2 for trimmed spectra
230319 HCN : add colors dictionary + P_diss correction
230318 HCN : add f_Ohm and prefactors in laws of Christensen
230315 HCN : add triming of spectra with symmetry-induced wiggles
230314 HCN : complete reorganization...
230313 HCN : modif to plot viscous and ohmic dissipation separately
230307 HCN : modif to operate on S(n) spectra
221207 HCN : version to plot tau-ell diagram of DNS simulation S2 (Schaeffer et al, 2017)
221107 HCN : add tau_ff(l) assuming AC balance at large scale
221007 HCN : add computation of E_M/E_K ratio from Davidson (2013)
221002 HCN : use length_nu = length.copy() to create new array in memory
"""

from read_parameters import read_parameters
from plot_template import plot_template
import tau_ell_lib as tl
import numpy as np
import matplotlib.pyplot as plt

# adjust these parameters :
# >>>>>>>>>>>>>>>>>>>>>>>>>
# name of 'object' in read_parameters
object = 'S2'
#object = 'Guervilly'
#object = 'Dormy_strong'
# do consider magnetic field
with_mag = True
# ****
stress_free = False
# ****
# template flag if no u, b or rho spectra wanted
template = False
# flag to plot circles and dimensionless number label at line intersections
plot_dimensionless = True
# flag to plot scaling laws of Christensen (2010) and Davidson (2013)
plot_scaling_laws = False
# plot ell_c, tau_c, tau_rho_c at convection onset
plot_convection_onset = False
# flag for secondary time axis in years
y_sec = True
# IC/OC volume ratio limit for shell_case
IC_over_OC_limit = 0.8 # just right to get the Sun convective zone as a shell
# <<<<<<<<<<<<<<<<<<<<<<<<<
#
title = object + ' DNS turbulence'
#
# --> get needed object's parameters
# ----------------------------------
param = read_parameters(object)
print(param)
R_o = param['R_o']
R_i = param['R_i']
nu = param['nu']
kappa = param['kappa']
gravity = param['gravity']
#rho = param['rho']
object_type = param['object_type']
#
# a few auxiliary parameters
DD = R_o-R_i # [m] shell thickness
#
# flag for both radial and non-radial velocities
try:
    dumy = param['file_spec_u_r']
    flag_anis = True
except KeyError:
    flag_anis = False
# flag for magnetic case
try:
    dumy = param['file_spec_b']
    flag_b = True
except KeyError:
    flag_b = False
#
# --> adjust min and max time and lengths below
# ---------------------------------------------
if object_type == 'star_sim':
# --> min and max lengths
    xmin, xmax = 1e6, 1 # SunSim (will be changed into xmax = (R_o+R_i)/2 xmax)
# --> min and max times
    ymin, ymax = 1e3, 1e8 # SunSim
elif object_type == 'planetary_core_sim':
    xmin, xmax = 1e3, 1 # (will be changed into xmax = (R_o+R_i)/2 xmax)
    ymin, ymax = 1e4, 1e9 # S2
else:
    xmin, xmax = 1e-2, 1 # (will be changed into xmax = (R_o+R_i)/2 xmax)
    ymin, ymax = 1e3, 1e14 # 
#
# depending on whether the largest volume is outside or inside the tangent cylinder (TC),
# we consider DD=R_o or DD=R_o-R_i as our maximum depth and R_av=(R_o+R_i)/2 as max length-scale
shell_case = tl.select_shell(R_o,R_i,IC_over_OC_limit)
if shell_case:
    xmax = xmax * (R_o+R_i)/2
else:
    xmax = xmax * R_o   
#
# shifts for labels
shift = 1.2
t_symb = ymax*0.1
# -----------------------------------------------------------------
# plot the template
# -----------------------------------------------------------------
fig, ax, colors = plot_template(object, template, xmin, xmax, ymin, ymax, y_sec, with_mag, stress_free, shell_case, plot_dimensionless, plot_scaling_laws, plot_convection_onset)
# -----------------------------------------------------------------
#
# =========================================================================== #
#                                                                             #
#                      plot DNS spectra when available                        #
#                                                                             #
# =========================================================================== #
if not template:
# update title
    if with_mag:
        title = object + ' DNS dynamo turbulence'
    else:
        title = object + ' DNS turbulence'
    ax.set(title=title + '\n')
# parameters to be used in the conversion of spectra
    param_DNS = {}
#param_DNS.update({'rho':param['rho']}) # NB : cancels out in the end
# replace R_o by volumetric average of radius Rave for degree --> ell conversion
    R_i = R_o-DD
    Rave = 3/4 * (R_o**4-R_i**4)/(R_o**3-R_i**3)
    param_DNS.update({'R_o':Rave})
# replace gravity at R_o by gravity at Rave
    param_DNS.update({'gravity':Rave/R_o*param['gravity']})
# -----------------------------------------------------------------------
# read and plot u spectrum obtained by Nathanael from his DNS simulations
# -----------------------------------------------------------------------
    try:
        filename = param['file_spec_u']
# read file
        f = open(filename, 'r', encoding='utf8')
# header : 2 lines
        header = f.readline()
        header = f.readline()
#
        contents = f.readlines()
        print('---> ' + str(len(contents)) + ' lines read from ' + filename)
# decode file
        list = [] # create list
#
        degree = np.arange(1,len(contents)+1,1)
#
        for ii in range(len(contents)):
            line = contents.pop(0)
            truc =line.split()
            list.append(float(truc[0]))
#
        spec = np.array(list)
# conversion from Re^2 Nath's spectrum to S_n
        spec = spec*nu**2/DD**2
# convert spectrum into (ell,tau)
        (OK, l_u_DNS, t_u_DNS) = tl.tauell_from_spectrum('u', 'DNS', degree, spec, param_DNS) 
        f.close()
#
# u : remove even degree points if spec_trim present
        try:
            spec_trim = param['spec_trim']
            print(' *** remove even degrees from velocity spectrum ***')
            inds_to_keep = np.array(range(0, len(l_u_DNS), 2))
            l_u_DNS = l_u_DNS[inds_to_keep]
            t_u_DNS = t_u_DNS[inds_to_keep]
        except KeyError:
            print('...')
#
        t_SV = t_u_DNS[0]
        if flag_anis:
            ax.plot(l_u_DNS, t_u_DNS, color=colors['u'], label='$u$', linewidth=4, alpha=0.2) # non-radial velocity line
            ax.plot(R_o, t_SV, 'o', color=colors['u'], markersize=10, clip_on=False, alpha = 0.3)
#            plt.text(R_o*shift, t_SV, "$t_u^{obs}$", va='center', color=colors['u'])
        else:
            ax.plot(l_u_DNS, t_u_DNS, color=colors['u'], label='$u$', linewidth=4, alpha=0.5) # velocity line
            ax.plot(R_o, t_SV, 'o', color=colors['u'], markersize=10, clip_on=False)
#            plt.text(R_o*shift, t_SV, "$t_u^{obs}$", va='center', color=colors['u'])
##            plt.text(R_o*shift, t_SV, "$t_u^{obs}$", va='bottom', color=colors['u'])
#
    except KeyError:
        OK = False
#
# -----------------------------------------------------------------------
# read and plot u_r spectrum obtained by Nathanael from his DNS simulations
# -----------------------------------------------------------------------
    try:
        filename = param['file_spec_u_r']
# read file
        f = open(filename, 'r', encoding='utf8')
# header : 2 lines
        header = f.readline()
        header = f.readline()
#
        contents = f.readlines()
        print('---> ' + str(len(contents)) + ' lines read from ' + filename)
# decode file
        list = [] # create list
#
        degree = np.arange(1,len(contents)+1,1)
#
        for ii in range(len(contents)):
            line = contents.pop(0)
            truc =line.split()
            list.append(float(truc[0]))
#
        spec = np.array(list)
# conversion from Re^2 Nath's spectrum to S_n
        spec = spec*nu**2/DD**2
# convert spectrum into (ell,tau)
        (OK, l_ur_DNS, t_ur_DNS) = tl.tauell_from_spectrum('u', 'DNS', degree, spec, param_DNS) 
        f.close()
    #
# u : remove even degree points if spec_trim present
        try:
            spec_trim = param['spec_trim']
            print(' *** remove even degrees from velocity spectrum ***')
            inds_to_keep = np.array(range(0, len(l_u_DNS), 2))
            l_ur_DNS = l_ur_DNS[inds_to_keep]
            t_ur_DNS = t_ur_DNS[inds_to_keep]
        except KeyError:
            print('...')
#
        ax.plot(l_ur_DNS, t_ur_DNS, color=colors['u'], label='$u_r$', linewidth=4, alpha=0.5) # radial velocity line
#            t_SV = t_u_DNS[0]
#            ax.plot(R_o, t_SV, 'o', color=colors['u'], markersize=10, clip_on=False)
#            plt.text(R_o*shift, t_SV, "$t_{SV}$", va='center', color=colors['u'])
#
# compute the estimated viscous dissipation from sum of 1/tau_u^2(ell)
#            eps = sum(t_u_DNS**(-2))
#            P_diss_nu_DNS = 2*nu*M_o*eps
#            print('....... P_diss_nu_DNS = ' + str(P_diss_nu_DNS) + ' W')
#            t_diss_nu_DNS = np.sqrt(nu*M_o/P_diss_nu_DNS)
#            l_diss_nu_DNS = np.sqrt(nu*t_diss_nu_DNS)
#            ax.plot([l_diss_nu_DNS, 5*l_diss_nu_DNS], [t_diss_nu_DNS, t_diss_nu_DNS], ':', color=colors['nu']) # P_diss_nu_DNS mark
    except KeyError:
        OK = False
#
# -----------------------------------------------------------------------
# read and plot b spectrum obtained by Nathanael from his DNS simulations
# -----------------------------------------------------------------------
    try:
        filename = param['file_spec_b']
# read file
        f = open(filename, 'r', encoding='utf8')
# header : 2 lines
        header = f.readline()
        header = f.readline()
# 
        contents = f.readlines()
        print('---> ' + str(len(contents)) + ' lines read from ' + filename)
# decode file
        list = [] # create list
#
        degree = np.arange(1,len(contents)+1,1)
#
        for ii in range(len(contents)):
            line = contents.pop(0)
            truc =line.split()
            list.append(float(truc[0]))
#
        spec = np.array(list)
# conversion from Alfven Re^2 Nath's spectrum to S_n
        spec = spec*nu**2/DD**2
# convert spectrum into (ell,tau)
        OK, l_b_DNS, t_b_DNS = tl.tauell_from_spectrum('b', 'DNS', degree, spec, param_DNS) 
        f.close()
#
# b : remove odd degree points if spec_trim present
        try:
            spec_trim = param['spec_trim']
            print(' *** remove odd degrees from magnetic spectrum ***')
            inds_to_keep = np.array(range(1, len(l_b_DNS), 2))
            l_b_DNS = l_b_DNS[inds_to_keep]
            t_b_DNS = t_b_DNS[inds_to_keep]
        except KeyError:
            print('...')
#
        ax.plot(l_b_DNS, t_b_DNS, color=colors['b'], label='$b$', linewidth=4, alpha=0.5) # magnetic line
        t_A0 = t_b_DNS[0]
        ax.plot(R_o, t_A0, 'o', color=colors['b'], markersize=10,clip_on=False)
##        plt.text(R_o*shift, t_A0, "$t_b^{obs}$", va='center', color=colors['b'])
#        plt.text(R_o*shift, t_A0, "$t_b^{obs}$", va='center', color=colors['b'])
#
    except KeyError:
        OK = False
#        l_b_DNS = float("NaN")
#        t_b_DNS = float("NaN")
#
# -----------------------------------------------------------------------
# read and plot rho spectrum obtained by Nathanael from his DNS simulations
# -----------------------------------------------------------------------
    try:
        filename = param['file_spec_rho']
# read file
        f = open(filename, 'r', encoding='utf8')
# header : 2 lines
        header = f.readline()
        header = f.readline()
#
        contents = f.readlines()
        print('---> ' + str(len(contents)) + ' lines read from ' + filename)
# decode file
        list = [] # create list
#
        degree = np.arange(1,len(contents)+1,1)
#
        for ii in range(len(contents)):
            line = contents.pop(0)
            truc =line.split()
            list.append(float(truc[0]))
#
        spec = np.array(list)
# conversion from (C)^2 Nath's spectrum to (Delta_rho/rho)^2
        spec = spec * (nu/kappa)**2
# convert spectrum into (ell,tau)
        OK, l_ff_DNS, t_ff_DNS = tl.tauell_from_spectrum('rho', 'DNS', degree, spec, param_DNS) 
        f.close()
#
# rho : remove odd degree points if spec_trim present
        try:
            spec_trim = param['spec_trim']
            print(' *** remove odd degrees from density spectrum ***')
            inds_to_keep = np.array(range(1, len(l_ff_DNS), 2))
            l_ff_DNS = l_ff_DNS[inds_to_keep]
            t_ff_DNS = t_ff_DNS[inds_to_keep]
        except KeyError:
            print('...')
#
        ax.plot(l_ff_DNS, t_ff_DNS, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5) # free-fall buoyancy line
        t_ff_0 = t_ff_DNS[0]
        ax.plot(R_o, t_ff_0, 'o', color=colors['rho'], markersize=10, clip_on=False)
#        plt.text(R_o*shift, t_ff_0, "$t_\rho^{obs}$", va='center', color=colors['rho'])
##        plt.text(R_o*shift, t_ff_0, "$t_\rho^{obs}$", va='bottom', color=colors['rho'])
    except KeyError:
        OK = False
#        l_ff_DNS = float("NaN")
#        t_ff_DNS = float("NaN")
#
    ax.grid(visible=None)
# -------------------------
ax.legend(loc='upper left')

plt.show()
