#!/opt/anaconda3/bin/python
"""
==========================
Kolmogorov tau-ell diagram
==========================
Small script to plot a tau-ell regime diagram for standard Kolmogorov behaviour
From Treatise on Geophysics 'Turbulence in the core' chapter
Nataf H-C. and N. Schaeffer, 2015.

230319 HCN : reorganization + add colors dictionary
221001 HCN : anchor to P_diss on the viscous line
220925 HCN : translate matlab pgms into python
"""

from read_parameters import read_parameters
import tau_ell_lib as tl
import numpy as np
import matplotlib.pyplot as plt

# adjust these parameters :
# -------------------------
# name of 'object' in read_parameters
object = 'Earth'
# flag to plot circles and dimensionless number label at line intersections
plot_dimensionless = True
# flag for secondary time axis in years
y_sec = True
# --> min and max lengths
xmin, xmax = 1e-2, 1 # (will be changed into xmax = R_o*xmax)
# --> min and max times
ymin, ymax = 10, 1e12
# shifts for labels
shift = 1.3
t_symb = ymax*0.1
# -------------------------
#
title = object +' Kolmogorov universal turbulence'
#
param = read_parameters(object)
print(param)
R_o = param['R_o']
xmax = xmax*R_o
R_i = param['R_i']
M_o = param['M_o']
nu = param['nu']
P_diss = param['P_diss']
rho = param['rho']
t_SV = param['t_SV']
#
# a few auxiliary parameters
dd = R_o-R_i # [m] shell thickness

#
# launch the figure
# ---------------------------------------------------------
fig, ax, colors = tl.tau_ell_setup(title,xmin,xmax,ymin,ymax,y_sec)
# ---------------------------------------------------------
# construct length vector
pmin = np.floor(np.log10(xmin))
pmax = np.ceil(np.log10(xmax))
length = 10**np.arange(pmin,pmax,1.0) # [m] length vector
length[-1] = xmax
#
# construct and draw lines from parameters
# -----------------------------------------
plt.text(R_o, ymin*0.6, "$R_o$", ha='center', va='top')
#
# --- construct and draw viscous diffusion line
length_nu = length.copy() # viscosity length will be trimed above spinup transition
t_nu = length_nu**2/nu # [s]
ax.plot(length_nu, t_nu, color=colors['nu']) # viscous diffusion line
plt.text(np.sqrt(nu*t_symb*0.2)*0.8, t_symb*0.2, "$\\nu$", ha = 'right', va='bottom', color=colors['nu'])
# ----------- construct and draw viscous dissipated power markers
(l_nu_power, t_nu_power, index, label) = tl.construct_power_markers(nu, M_o)
# plot bulk viscous dissipation markers
ax.plot(l_nu_power, t_nu_power, 's', color=colors['nu'], fillstyle='none', markersize=12)
# label selected dissipation markers
for exp in (0, 3, 6, 9, 12):
    ind = index[exp]
    l_exp = l_nu_power[ind]
    t_exp = t_nu_power[ind]
    if exp == 3:
        ax.plot(l_exp, t_exp, 's', color=colors['nu'], markersize=12, alpha=0.2)
    if exp == 12:
        ax.plot(l_exp, t_exp, 's', color=colors['nu'], markersize=12)
    if (l_exp>xmin and l_exp<xmax):
        plt.text(l_exp*0.7, t_exp, label[exp], ha = 'right', va='bottom', color=colors['nu'], size=max(6,exp))

# --- construct Kolmogorov time vector anchored on P_diss
l_Kolmogorov, t_Kolmogorov = [], [] # create lists
# first point = point with P=P_diss on the viscous line
ttt = np.sqrt(nu*M_o/P_diss)
lll = np.sqrt(nu*ttt)
l_Kolmogorov.append(lll)
t_Kolmogorov.append(ttt)
# second point at R_o
l_Kolmogorov.append(R_o)
ttt = ttt*(R_o/lll)**(2./3) # time at R_o scale following k^-5/3 energy law
t_Kolmogorov.append(ttt)
ax.plot(l_Kolmogorov, t_Kolmogorov, color=colors['u'], label='$u$', linewidth=4, alpha=0.5) # velocity line
# ----- Reynolds number
if plot_dimensionless:
    plt.text(l_Kolmogorov[0],t_Kolmogorov[0],"$\\bigcirc$", size=20, ha='center', va='center')
    plt.text(l_Kolmogorov[0]*shift,t_Kolmogorov[0]/shift,"$\mathrm{Re}(\ell) \\sim 1$", size=18, ha='left', va='top')
#
# ---- plot dissipation line
t_diss = np.sqrt(nu*M_o/P_diss)
l_diss = np.sqrt(nu*t_diss)
ax.plot([l_diss, 10*l_diss], [t_diss, t_diss], '--', color=colors['nu']) # P_diss mark
#
ax.legend(loc='upper left')
ax.grid()

plt.show()
