#!/usr/bin/python3.9
#!/opt/anaconda3/bin/python
"""
=================================================
plot rotating convection tau-ell diagram template
=================================================
Small program to plot a tau-ell regime diagram for rotating convection
From Treatise on Geophysics 'Turbulence in the core' chapter
Nataf H-C. and N. Schaeffer, 2015.
Re-designed from Guervilly et al (2019) findings

230327 HCN : add wavy line for Rossby
230319 HCN : reorganization + new power markers + colors
221217 HCN : specialize for convection onsets
221204 HCN : correct expressions for convection onset
221002 HCN : type \\rho and \\nu for greek letters to avoid \r and \n specific meanings
"""

from read_parameters import read_parameters
import tau_ell_lib as tl
import numpy as np
import matplotlib.pyplot as plt

# adjust these parameters :
# -------------------------
# name of 'object' in read_parameters
object = 'Earth'
# flag for secondary time axis in years
y_sec = True
# --> min and max lengths
xmin, xmax = 1e-2, 1 # (will be changed into xmax = R_o*xmax)
# --> min and max times
ymin, ymax = 1e4, 1e20
# shifts for labels
shift = 1.3
t_symb = ymax*0.0001
# -------------------------
#
title = object +' convection onsets'
#
param = read_parameters(object)
print(param)
R_o = param['R_o']
xmax = xmax*R_o
R_i = param['R_i']
Omega = param['Omega']
M_o = param['M_o']
nu = param['nu']
kappa = param['kappa']
kappa_chi = param['kappa_chi']
eta = param['eta']
P_diss = param['P_diss']
gravity = param['gravity']
alpha = param['alpha']
C_p = param['C_p']
rho = param['rho']
t_SV = param['t_SV']
t_A0 = param['t_A0']
#
# a few auxiliary parameters
dd = R_o-R_i # [m] shell thickness
HH = C_p/gravity/alpha # [m] scale height
t_omega = 1./Omega # [s] Omega^(-1) reference time
l_Ekman = np.sqrt(nu/Omega) # [m] Ekman layer thickness
Ekman = nu/Omega/R_o**2 # Ekman number
t_spinup = R_o/np.sqrt(nu*Omega) # [s] spinup time
l_spinup = R_o * Ekman**(1./4.) # [m] length for transition to spinup
#
# launch the figure
# ---------------------------------------------------------
fig, ax, colors = tl.tau_ell_setup(title,xmin,xmax,ymin,ymax,y_sec)
# ---------------------------------------------------------
# construct length vector
pmin = np.floor(np.log10(xmin))
pmax = np.ceil(np.log10(xmax))
length = 10**np.arange(pmin,pmax,1.0) # [m] length vector
length[-1] = xmax
#
# construct and draw lines from parameters
# -----------------------------------------
plt.text(R_o, ymin*0.2, "$R_o$", ha='center', va='top')
ax.plot(HH, ymin, 'ob', markersize=10)
if HH<xmax:
    plt.text(HH, ymin*0.4, "$H$", ha='center', va='top')
#
# ---- draw day line
ax.plot([xmin, R_o], [t_omega, t_omega], '-.', color=colors['day'],label='$t_\\Omega$')
plt.text(R_o*shift, t_omega, "$t_\Omega$", va='center', color=colors['day'])
#
# --- construct and draw thermal diffusion line
t_kappa = length**2/kappa # [s]
ax.plot(length, t_kappa, color=colors['kappa'])
plt.text(np.sqrt(kappa*t_symb)/0.7, t_symb, "$\\kappa$", ha = 'left', va='top', color=colors['kappa'])
#
# --- construct and draw chemical diffusion line
t_kappa_chi = length**2/kappa_chi ; # [s]
ax.plot(length, t_kappa_chi, color=colors['chi'])
plt.text(np.sqrt(kappa_chi*t_symb)*0.8, t_symb, "$\\chi$", ha = 'right', va='bottom', color=colors['chi'])
#
# --- construct and draw viscous diffusion line
length_nu = length.copy() # viscosity length will be trimed above spinup transition
t_nu = length_nu**2/nu # [s]
ax.plot(length_nu, t_nu, color=colors['nu']) # viscous diffusion line
ax.plot([l_spinup, R_o], [t_spinup, t_spinup], color=colors['nu']) # spinup line ***
plt.text(np.sqrt(nu*t_symb)*0.8, t_symb, "$\\nu$", ha = 'right', va='bottom', color=colors['nu'])
# ----------- construct and draw viscous dissipated power markers
(l_nu_power, t_nu_power, index, label) = tl.construct_power_markers(nu, M_o)
# plot bulk viscous dissipation markers
ax.plot(l_nu_power, t_nu_power, 's', color=colors['nu'], fillstyle='none', markersize=12)
# label selected dissipation markers
for exp in ( -9, 3, 9):
    ind = index[exp]
    l_exp = l_nu_power[ind]
    t_exp = t_nu_power[ind]
    if exp == 3:
        ax.plot(l_exp, t_exp, 's', color=colors['nu'], markersize=12, alpha=0.2)
    if exp == 12:
        ax.plot(l_exp, t_exp, 's', color=colors['nu'], markersize=12)
    if (l_exp>xmin and l_exp<xmax):
        plt.text(l_exp*0.5, t_exp, label[exp], ha = 'right', va='bottom', color=colors['nu'], size=max(9,exp))
#
# --- construct and draw Rossby line
l_Rossby, t_Rossby = [], [] # create lists
[lll, ttt] = tl.function_intersect(R_o, 1./Omega, -1., 1., 1./nu, 2) # intersection  Rossby time with avec t_nu
l_Rossby.append(lll) # first point
t_Rossby.append(ttt)
l_Rossby.append(R_o) # second point
t_Rossby.append(t_omega)
#ax.plot(l_Rossby, t_Rossby, ':', color=colors['Rossby'], label='$Rossby$') # Rossby line
line_kwargs = {'color':colors['Rossby'], 'alpha':0.5}
tl.plot_wave(l_Rossby, t_Rossby, npts=100, nper=20, famp=0.2, line_kwargs=line_kwargs)
# ----------- construct and draw QG viscous dissipated power markers
(l_nu_QG_power, t_nu_QG_power, index, label) = tl.construct_QG_power_markers(l_Ekman, Omega, R_o, M_o)
# delete QG power markers that fall beyond the viscous line (very pythonesq syntax!)
l_nu_power_plot, t_nu_power_plot = [], []
t_nu_power_plot[:] = [x for x in t_nu_QG_power if x<t_Rossby[0]]
l_nu_power_plot[:] = [x for x in l_nu_QG_power if x>l_Rossby[0]]
# plot Ekman layer viscous dissipation markers
ax.plot(l_nu_power_plot, t_nu_power_plot, 'd', color=colors['nu'], fillstyle='none', markersize=12)
# label selected QG dissipation markers
for exp in (3, 12):
    ind = index[exp]
    l_exp = l_nu_QG_power[ind]
    t_exp = t_nu_QG_power[ind]
    if exp == 3:
        ax.plot(l_exp, t_exp, 'd', color=colors['nu'], markersize=12, alpha=0.2)
    if exp == 12:
        ax.plot(l_exp, t_exp, 'd', color=colors['nu'], markersize=12)
    if (l_exp>xmin and l_exp<xmax):
        plt.text(l_exp*0.5, t_exp, label[exp], ha = 'right', va='top',  color=colors['nu'], size=max(9,exp))

# plot and plot rotating convection onset markers
# -----------------------------------------------
# --- construct (tau=period,ell=wavelength) and free-fall point at convection threshold
# following local marginal stability theory of Busse (1970). His equations (4.11) to (4.13).
#
# +++ using thermal diffusivity ++++
# intersection  Rossby line with first encountered of t_nu(ell) or t_kappa(ell)
[lll_c, t_temp] = tl.function_intersect(R_o, 1./Omega, -1., 1., 1./max(nu,kappa), 2)
# lll_c is the column width at threshold
# ttt_c is the drift period at threshold = t_kappa(lll_c)
ttt_c = lll_c**2/kappa
# t_rho_c at ell = R_o is the critical tau_rho (from critical Ra)
# at mid-distance between t_nu(lll_c) and t_kappa(lll_c)
t_ff_c = np.sqrt(lll_c**2/nu * lll_c**2/kappa)
ax.plot(lll_c,ttt_c, 'vk',markersize=12) # (tau_c,l_c) at convection threshold
ax.plot([lll_c, R_o],[t_ff_c, t_ff_c], ':', color = colors['rho'])
ax.plot(R_o,t_ff_c, 'v', color = colors['rho'], markersize=12, clip_on=False) # (R_o, t_ff_c) tau_rho(R_o) at convection threshold
#
# ++++ using compositional diffusivity ++++
[lll_chi, t_temp] = tl.function_intersect(R_o, 1./Omega, -1., 1., 1./max(nu,kappa_chi), 2)
# lll_c is the column width at threshold
# ttt_c is the drift period at threshold = t_kappa_chi(lll_c)
ttt_chi = lll_chi**2/kappa_chi
ttt_chi_temp = t_temp 
# t_rho_c at ell = R_o is the critical tau_rho (from critical Ra)
# at mid-distance between t_nu(lll_c) and t_kappa_chi(lll_c)
t_ff_chi = np.sqrt(lll_chi**2/nu * lll_chi**2/kappa_chi)
ax.plot(lll_chi,ttt_chi, '^k',markersize=12) # (tau_chi,l_chi) at convection threshold
ax.plot([lll_chi, lll_chi],[ttt_chi_temp, ttt_chi], ':', color = colors['chi'])
ax.plot([lll_chi, R_o],[t_ff_chi, t_ff_chi], ':', color = colors['rho'])
ax.plot(R_o,t_ff_chi, '^', color = colors['rho'], markersize=12, clip_on=False) # (R_o, t_ff_chi) tau_rho(R_o) at convection threshold
#
# plot non-rotating convection onset markers
# ------------------------------------------
# ---- non-rotating convection onset (Ra ~ 1 => tau_rho = sqrt(tau_nu tau_kappa) (thermal)
t_ff_Rayleigh = np.sqrt(R_o**2/nu * R_o**2/kappa)
# non-rotating convection
ax.plot(R_o,t_ff_Rayleigh, 'o', color=colors['rho'], markersize=12, clip_on=False) # (R_o, t_ff_Rayleigh) tau_rho(R_o) at convection threshold
#
ax.legend(loc='upper left')
#ax.grid()

plt.show()
