#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
==========================================
plot Laboratory experiment tau-ell diagram
==========================================
Small script to plot a tau-ell regime diagram for rotating convection
From Treatise on Geophysics 'Turbulence in the core' chapter
Nataf H-C. and N. Schaeffer, 2015.

240206 HCN : adapt to shell and get UDV as k-spectra instead of tau-ell
230327 HCN : add wavy lines for Rossby and Alfven
230319 HCN : complete reorganization + new power markers + add colors dictionary
230219 HCN : for DTS : f-spectrum of u + k-spectrum of b (correct mT --> T)
221207 HCN : version to plot tau-ell diagram of Lab experiments
221107 HCN : add tau_ff(l) assuming AC balance at large scale
221007 HCN : add computation of E_M/E_K ratio from Davidson (2013)
221002 HCN : use length_nu = length.copy() to create new array in memory
"""

from read_parameters import read_parameters
from plot_template import plot_template
import tau_ell_lib as tl
import numpy as np
import matplotlib.pyplot as plt
#
mu0 = 4e-7*np.pi # SI magnetic permeability
#
# adjust these parameters :
# >>>>>>>>>>>>>>>>>>>>>>>>>
# name of 'object' in read_parameters
object = 'DTS'
# do consider magnetic field
with_mag = True
# meachanical boundary condition
stress_free = False
# template flag if no u, b or rho spectra wanted
template = False
# plot tau_u(R_o) and tau_b(R_o) from scaling laws (Christensen, 2010; Davidson, 2013)
plot_scaling_laws = False
# plot ell_c, tau_c, tau_rho_c at convection onset
plot_convection_onset = False
# flag to plot circles and dimensionless number label at line intersections
plot_dimensionless = False
# flag for secondary time axis in years
y_sec = False
# IC/OC volume ratio limit for shell_case
IC_over_OC_limit = 0.8 # just right to get the Sun convective zone as a shell
# <<<<<<<<<<<<<<<<<<<<<<<<<
#
title = object + ' experiment turbulence'
#
# --> get needed object's parameters
# ----------------------------------
param = read_parameters(object)
print(param)
R_o = param['R_o']
R_i = param['R_i']
rho = param['rho']
t_SV = param['t_SV']
t_A0 = param['t_A0']
object_type = param['object_type']
#
# --> adjust min and max time and lengths below
# ---------------------------------------------
# --> min and max lengths
if object_type == 'experiment':
    xmin, xmax = 1e-6, 1 # (will be changed into xmax = R_o*xmax)
# --> min and max times
    ymin, ymax = 1e-4, 1e3
else:
    xmin, xmax = 1e-4, 1 # Sun (will be changed into xmax = (R_o+R_i)/2 xmax)
    ymin, ymax = 1e-2, 1e12 # Sun
#
# depending on whether the largest volume is outside or inside the tangent cylinder (TC),
# we consider DD=R_o or DD=R_o-R_i as our maximum depth and R_av=(R_o+R_i)/2 as max length-scale
shell_case = tl.select_shell(R_o,R_i,IC_over_OC_limit)
if shell_case:
    DD = R_o-R_i # [m] shell thickness
    xmax = xmax * (R_o+R_i)/2
else:
    DD = R_o
    xmax = xmax * R_o
#
# shifts for labels
shift = 1.2
t_symb = ymax*0.1
# -----------------------------------------------------------------
# plot the template
# -----------------------------------------------------------------
fig, ax, colors = plot_template(object, template, xmin, xmax, ymin, ymax, y_sec, with_mag, stress_free, shell_case, plot_dimensionless, plot_scaling_laws, plot_convection_onset)
# -----------------------------------------------------------------
#
# =========================================================================== #
#                                                                             #
#                 plot experimental spectra when available                    #
#                                                                             #
# =========================================================================== #
if not template:
# update title
    if with_mag:
        title = object + ' experiment MHD turbulence'
    else:
        title = object + ' experiment turbulence'
    ax.set(title=title + '\n')
# parameters to be used in the conversion of spectra
    param_Lab = {'R_o':R_o}
#
# read and plot u spectrum obtained from UDV frequency spectrum
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# DTS Lab experiment : vortex overturn time
# kinetic energy times from the frequency spectrum of Doppler file48 of 060221
# f_o = 5 Hz, Df = 10 Hz, 40s-long set of aimuthal velocity profiles
# data extracted with UDV, k-spectrum computed with plot_UDV_spectrum.m
# depth = 60:80mm (indices 93:125), moving average on 10 tau points
#
    try:
        filename = param['file_spec_u']
# replace R_o by R_o-depth
        param_Lab.update({'R_o':R_o-0.07})
# read file
        f = open(filename, 'r', encoding='utf8')
# header : 2 lines
        header = f.readline()
        header = f.readline()
# spectrum : 2 columns : k, spectrum
        contents = f.readlines()
        print('---> ' + str(len(contents)) + ' lines read from ' + filename)
        k_list, spec_list = [], [] # create lists
        for ii in range(len(contents)):
            line = contents.pop(0)
            truc =line.split()
            k_list.append(float(truc[0]))
            spec_list.append(float(truc[1]))
#
        kk = np.array(k_list) # [m^-1]
        spec = np.array(spec_list) # [m^3/s^2]
# convert spectrum into (ell,tau)
        (OK, l_u_Lab, t_u_Lab) = tl.tauell_from_spectrum('u', 'K41', kk, spec, param_Lab)
        f.close()
    except KeyError:
        OK = False
    if OK:
        ax.plot(l_u_Lab, t_u_Lab, color=colors['u'], label='$u$', linewidth=4, alpha=0.5) # velocity line
#
# read and plot b spectrum obtained from magnetic wavenumber spectrum
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# DTS Lab experiment :
# magnetic time (from k-spectra)
# 150708S6X11 comp BB on meridian Q, 60s-long records
# f_o = 10.12, Deltaf = 9.95
#
    try:
        filename = param['file_spec_b']
# surface --> R_o
        param_Lab.update({'R_o':R_o})
# read file
        f = open(filename, 'r', encoding='utf8')
# header : 2 lines
        header = f.readline()
        header = f.readline()
# spectrum : 2 columns : k, spectrum
        contents = f.readlines()
        print('---> ' + str(len(contents)) + ' lines read from ' + filename)
# decode file
        k_list, spec_list = [], [] # create lists
        for ii in range(len(contents)):
            line = contents.pop(0)
            truc =line.split()
            k_list.append(float(truc[0]))
            spec_list.append(float(truc[1]))
#
        kk = np.array(k_list)
        spec = np.array(spec_list)/rho/mu0/1e6 # [m^3/s^2] after converting mT^2 to m^2/s^2

# convert spectrum into (ell,tau)
        (OK, l_b_Lab, t_b_Lab) = tl.tauell_from_spectrum('b', 'K41', kk, spec, param_Lab)
        f.close()
    except KeyError:
        OK = False
    if OK:
        ax.plot(l_b_Lab, t_b_Lab, color=colors['b'], label='$b$', linewidth=4, alpha=0.5) # velocity line
    try:
        t_b0 = param['t_b0'] # added for DTS experiment
        ax.plot(R_o, t_b0, 'o', color=colors['b'], markersize=6, clip_on=False)
        plt.text(R_o*shift, t_b0, "$t_{b}$", va='center', color=colors['b'])
    except KeyError:
        print('...')
    ax.grid(visible=None)
# -------------------------
ax.legend(loc='upper left')
#fig.tight_layout()
plt.show()
