#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
==============================================
plot tau-ell regime diagram of a given object
==============================================
Small script to plot a tau-ell regime diagram for rotating convection in a spherical shell
From 'Dynamic regimes in planetary cores: tau-ell diagrams', Nataf H-C. and N. Schaeffer, Compte-rendu Geosciences (2024).

240206 HCN : small adjustments
240205 HCN : small adjustments taken from original tau-ell_programs
240130 HCN : small adjustments to plot full sphere (not completed yet)
231215 HCN : complete rewriting ** pb P_diss arrows ***
231204 HCN : install observed tau(ell) for u and rho
231130 HCN : remove QG markers, add P_diss arrow on viscous line
230515 HCN : correct bu plot QG markers
230424 HCN : start shell version
230327 HCN : add wavy line for Rossby
230321 HCN : modify tau_rho line --> flat
230319 HCN : add colors dictionary
230314 HCN : complete reorganization...
230201 HCN : correct translation tau(ell) from Energy spectrum from numerical simu
221204 HCN : correct expressions for convection onset
221106 HCN : extend tau_ff lines to thermal dissipation line
221011 HCN : add tau_ff at threshold
221006 HCN : esthetic modifs
221002 HCN : type \\rho and \\nu for greek letters to avoid \r and \n specific meanings
"""

from read_parameters import read_parameters
from plot_template import plot_template
from plot_scenario import plot_scenario
import tau_ell_lib as tl
#import numpy as np
import matplotlib.pyplot as plt

# adjust these parameters :
# >>>>>>>>>>>>>>>>>>>>>>>>>
# name of 'object' in read_parameters
#object = 'Europa ocean'
#object = 'Ganymede'
object = 'Earth'
#object = 'Venus'
#object = 'GI846' 
#object = 'GI382' 
#object = 'GJ1246' 
#object = 'GI408' 
#object = 'Magma ocean' 
#object = 'Aubert_path_100'
#force_balance = 'QG-CIA' # QG-CIA balance at l_*
#force_balance = 'QG-VAC' # QG-VAC balance at l_c
#force_balance = 'IMAC' # IMAC balance scenario
#force_balance = 'CIA' # CIA balance scenario
#force_balance = 'MAC' # MAC balance scenario
#force_balance = 'QG-MAC' # QG-MAC balance scenario at l_perp
#force_balance = 'QG-MACa' # alternative QG-MAC balance scenario at l_perp
force_balance = 'QG-MAC_JA' # QG-MAC balance scenario at l_perp + constant vorticity a la Aubert
l_perp_over_R_o = 1/10 # relative scale at which force-balance applies
# do consider magnetic field
with_mag = True
# meachanical boundary condition
stress_free = False
# template True if no u, b or rho spectra wanted
template = False
# plot and label line intersections with dimensionless numbers
plot_dimensionless = True
# plot tau_u(R_o) and tau_b(R_o) from scaling laws (Christensen, 2010; Davidson, 2013)
plot_scaling_laws = False
# plot ell_c, tau_c, tau_rho_c at convection onset
plot_convection_onset = False
# flag for secondary time axis in years
y_sec = True
# IC/OC volume ratio limit for shell_case
IC_over_OC_limit = 0.8 # just right to get the Sun convective zone as a shell
# <<<<<<<<<<<<<<<<<<<<<<<<<
#
# --> get needed object's parameters
# ----------------------------------
param = read_parameters(object)
print(param)
R_o = param['R_o']
R_i = param['R_i']
object_type = param['object_type']
#
l_perp = R_o * l_perp_over_R_o # [m] force-balance length-scale
#
# --> adjust min and max time and lengths below if needed
# -------------------------------------------------------
(xmin, xmax, ymin, ymax) = tl.get_typical_limits(object_type)
# NB: xmax=1 rescaled by:
# depending on whether the largest volume is outside or inside the tangent cylinder (TC),
# we consider DD=R_o or DD=R_o-R_i as our maximum depth and R_av=(R_o+R_i)/2 as max length-scale
shell_case = tl.select_shell(R_o,R_i,IC_over_OC_limit)
if shell_case:
    xmax = xmax * (R_o+R_i)/2
else:
    xmax = xmax * R_o   
# -----------------------------------------------------------------
# plot the template
# -----------------------------------------------------------------
fig, ax, colors = plot_template(object, template, xmin, xmax, ymin, ymax, y_sec, with_mag, stress_free, shell_case, plot_dimensionless, plot_scaling_laws, plot_convection_onset)
# -----------------------------------------------------------------
# -----------------------------------------------------------------
# plot the scenario
# -----------------------------------------------------------------
if not template:
    plot_scenario(object, force_balance, l_perp, ax, colors, ymin, ymax, y_sec, with_mag, stress_free, plot_dimensionless, shell_case)
# -----------------------------------------------------------------
#
fig.tight_layout()
plt.show()
