#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
====================================
plot tau-ell regime diagram scenario
====================================
Small function to plot a tau-ell scenario for rotating convective dynamo in a spherical shell
From 'Dynamic regimes in planetary cores: tau-ell diagrams', Nataf H-C. and N. Schaeffer, Compte-rendu Geosciences (2024).

Instructions:
------------
- one complete scenario per if condition
- do not modify existing scenario: create new scenario with name such as QG-MACb for a variant of QG-MAC
- intersection coordinates of tau_u(ell), tau_b(ell) and tau_rho(ell) with template's lines tau_x(ell)
  are written (l_u_x, t_u_x) etc where x is line's parameter (nu, eta, Rossby, Omega, etc). They
  are used to label dimensionless numbers.
- if a given scenario for a given object produces an inconsistent tau-ell diagram, try another scenario 

This function returns nothing

Currently implemented scenarios:
- CIA
- QG-CIA
- QG-VAC
- MAC
- QG-MAC, QG-MACa, QG-MAC_JA
- IMAC

240107 HCN : adjustments in IMAC scenario for Venus 
231215 HCN : new shell organization *** to be rewritten with arrays ***
"""

from read_parameters import read_parameters
import tau_ell_lib as tl
import numpy as np
import matplotlib.pyplot as plt

# ===========================================================================
def plot_scenario(object, force_balance, l_perp, ax, colors, ymin, ymax, y_sec, with_mag, stress_free, plot_dimensionless, shell_case):
# ===========================================================================
#
# update title
    if with_mag:
        title = object + ' ' + force_balance + ' rotating convective dynamo'
    else:
        title = object + ' ' + force_balance + ' rotating convective turbulence'
    ax.set(title=title + '\n')
#
# --> get needed object's parameters
# ----------------------------------
    param = read_parameters(object)
    R_o = param['R_o']
    R_i = param['R_i']
    Omega = param['Omega']
    M_o = param['M_o']
    nu = param['nu']
    kappa = param['kappa']
    kappa_chi = param['kappa_chi']
    eta = param['eta']
    P_diss = param['P_diss']
    gravity = param['gravity']
    alpha = param['alpha']
    C_p = param['C_p']
    rho = param['rho']
    t_SV = param['t_SV']
    t_A0 = param['t_A0']
    obs_list = param['obs_list']
    object_type = param['object_type']

# -------------------------------------------------------
# a few auxiliary parameters
#dd = R_o-R_i # [m] shell thickness
    R_av = (R_o+R_i)/2 # [m] shell mean radius
#V_shell = 4*np.pi/3 * (R_o**3 - R_i**3) # volume of the complete shell
    HH = C_p/gravity/alpha # [m] scale height
##
#if object_type == 'star_sim': # non-compressible simulations
#    HH = dd
    t_Omega = 1./Omega # [s] Omega^(-1) reference time
    l_Ekman = np.sqrt(nu/Omega) # [m] Ekman layer thickness
    Ekman = nu/Omega/(R_o-R_i)**2 # Ekman number
    t_spinup = (R_o-R_i)/np.sqrt(nu*Omega) # [s] spinup time *** check ?
    l_spinup = (R_o-R_i) * Ekman**(1./4.) # [m] length for transition to spinup *** check ?
#
# a few graphical stuff
    shift = 1.3 # shift for labels

# o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o #
#                                                                             #
#                            scenario encyclopedia                            #
#                                                                             #
# o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o #
# ================================================== #
# CIA balance scenario (Coriolis-Inertia-Archimedes) #
# ================================================== #
    if force_balance == 'CIA':
# get available power for viscous dissipation 
        try:
            P_diss_nu = param['P_diss_nu']
        except KeyError:
            P_diss_nu = P_diss
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# first point at P_diss_nu on viscous line (231206)
        t_u_nu = np.sqrt(M_o*nu/(P_diss_nu))
        l_u_nu = np.sqrt(t_u_nu*nu)
# plot arrow at viscous  dissipation on viscous diffusion line
        ax.annotate('',xy=(l_u_nu, t_u_nu), xytext=(l_u_nu/6, t_u_nu/2), arrowprops=dict(edgecolor=colors['nu'],facecolor=colors['nu']))
# for later use : get l_Omega of Rossby=1 (intersection of K41 with t_Omega)
        [l_u_Omega, t_u_Omega] = tl.function_intersect(1, t_Omega, 0, l_u_nu, t_u_nu, 2/3)
# for later use : get l_u_eta of Rm=1 (intersection of K41 with t_Omega)
        [l_u_eta, t_u_eta] = tl.function_intersect(1, 1/eta, 2, l_u_nu, t_u_nu, 2/3)
# second point K41 at injection scale = scale height H or R_o-R_i following K41 (2/3 slope)
        l_u_H = min(HH,R_o-R_i)
        t_u_H = t_u_nu*(l_u_H/l_u_nu)**(2/3)
# third point at intersection of slope 2/3 (K41) with Rossby line inside tangent cylinder 231206
        [l_u_Rossby, t_u_Rossby] = tl.function_intersect(R_i, t_Omega*(R_o-R_i)/R_i, -1., l_u_H, t_u_H, 2/3)
# fourth point at intersection with (r=R_o) with slope 2 (non-azimutal velocity) 231206
        l_u_R_o = R_o
        t_u_R_o = t_u_H*(R_o/l_u_H)**2
# adjustment if injection scale larger than l_Omega
        if l_u_H > l_u_Omega:
            l_u_H = l_u_Omega
            t_u_H = t_u_Omega
# create array of tau_u(ell) points
        l_u = np.array([l_u_nu, l_u_H, l_u_Omega, l_u_Rossby, l_u_R_o])
        t_u = np.array([t_u_nu, t_u_H, t_u_Omega, t_u_Rossby, t_u_R_o])
# draw force-balance length scale l_omega
        ax.plot([l_u_Omega, l_u_Omega], [ymin, ymax], dashes=(5,2,1,2,1,2,1,2), color=colors['day'], label='$\ell_\\Omega$')
# plot tau_u(ell) velocity line
        ax.plot(l_u, t_u, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at Ro=1 (intersection of line t_u with t_Omega)
        l_ff_Omega = l_u_Omega
        t_ff_Omega = t_Omega
# first point at intersection of kappa line with slope 1/2 tau_ff line
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_ff_Omega, t_ff_Omega, 1./2, 1., 1./kappa, 2)
# third point at intersection of line with slope 1/2 tau_ff with ell=R_o-R_i
        l_ff_R_o = R_o-R_i
        t_ff_R_o = t_ff_kappa * ((R_o-R_i)/l_ff_kappa)**0.5
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_ff_Omega, l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_ff_Omega, t_ff_R_o])
# plot tau_ff(ell) free-fall line
        ax.plot(l_ff, t_ff, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)

# ======================================================================= #
# QG-CIA balance scenario (Quasi-Geostrophic Coriolis-Inertia-Archimedes) #
# ======================================================================= #
    elif force_balance == 'QG-CIA':
# get available power for QG viscous dissipation 
        try:
            P_diss = param['P_diss_nu_QG']
        except KeyError:
            try:
                P_diss = param['P_diss_nu']
            except KeyError:
                P_diss = P_diss
# determine time t_* that yields P_diss QG viscous dissipation
        t_star = (M_o*R_o*l_Ekman/Omega/P_diss)**(1./4.)
# get corresponding l_* lengthscale on the Rossby line *** adapt to shell R_o or (R_o-R_i) ***
        l_star = R_o/Omega/t_star
# -----
# plot arrow at QG viscous dissipation on the Rossby line
        ax.annotate('',xy=(l_star, t_star), xytext=(l_star/6, t_star/2), arrowprops=dict(edgecolor=colors['nu'],facecolor=colors['nu']),color=colors['nu']) # P_diss arrow
# draw dissipation length scale l_star
        ax.plot([l_star, l_star], [ymin, ymax], dashes=(5,2,1,2,1,2), color = colors['nu'], label='$\ell_*$')
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# second point at (l_*,t_*) on the Rossby line
# first point at the intersection of viscous line and k^-5 line (-1 tau-ell slope)
        [l_u_nu, t_u_nu] = tl.function_intersect(l_star, t_star, -1, 1., 1./nu, 2)
# last point slope 2 to l=R_o 
        l_u_R_o = R_o
        t_u_R_o = t_star*(R_o/l_star)**2
# create array of tau_u(ell) points
        l_u = np.array([l_u_nu, l_star, l_u_R_o])
        t_u = np.array([t_u_nu, t_star, t_u_R_o])
# plot tau_u(ell) velocity line (* small shift for clarity)
        ax.plot(l_u, t_u, color=colors['u'], label='$u_r$', linewidth=4, alpha=0.2)
        ax.plot(l_u[0:2]*0.95, t_u[0:2]*0.95, color=colors['u'], linewidth=4, alpha=0.5)
#
# --- construct and draw tau_u_zonal(ell) velocity line
# -----------------------------------------------------
# first point at (l_*,t_*) on the Rossby line
# second point at l=R_o with slope 1
        t_u_zonal_R_o = t_star*(R_o/l_star)
# create array of tau_u_zonal(ell) points
        l_u_zonal = np.array([l_star, R_o])
        t_u_zonal = np.array([t_star, t_u_zonal_R_o])
# plot tau_u_zonal(ell) velocity line 
        ax.plot(l_u_zonal, t_u_zonal, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at l_star and t_star
# first point at intersection of thermal diffusion line and free-fall line (-1 tau-ell slope)
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_star, t_star, -1, 1., 1./kappa, 2)  
# third point at intersection of free-fall line (1/4 tau-ell slope) with l = R_o
        l_ff_R_o = R_o
        t_ff_R_o = (R_o/l_star)**(1/4)*t_star
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_star, l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_star, t_ff_R_o])
# plot tau_ff(ell) free-fall line (* small shift for clarity)
        ax.plot(l_ff*1.05, t_ff*1.05, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)
#
# --- construct and draw (tau=period,ell=wavelength) and free-fall point at convection threshold
# ----------------------------------------------------------------------------------------------
        if 0==0:
# following local marginal stability theory of Busse (1970). His equations (4.11) to (4.13).
# intersection  Rossby line with first encountered of t_nu(ell) or t_kappa(ell)
            [l_u_c, t_temp] = tl.function_intersect(R_o, 1./Omega, -1., 1., 1./max(nu,kappa), 2)
# l_u_c is the column width at threshold
# t_u_c is the drift period at threshold = t_kappa(lll_c)
            t_u_c = l_u_c**2/kappa
# t_rho_c at ell = R_o is the critical tau_rho (from critical Ra)
# at mid-distance between t_nu(lll_c) and t_kappa(lll_c)
            t_ff_c = np.sqrt(l_u_c**2/nu * l_u_c**2/kappa)
# plot rotating convection onset markers (thermal diffusivity)
            ax.plot(l_u_c,t_u_c, 'vk',markersize=12) # (tau_c,l_c) at convection threshold
            ax.plot([l_u_c, R_o],[t_ff_c, t_ff_c], ':', color=colors['rho']) # construction line
            ax.plot(R_o,t_ff_c, 'v', color=colors['rho'], markersize=12, clip_on=False) # (R_o, t_ff_c) tau_rho(R_o) at threshold
#
# ======================================================================= #
# QG-VAC balance scenario (Quasi-Geostrophic Viscous-Archimedes-Coriolis) #
# ======================================================================= #
    elif force_balance == 'QG-VAC':
# get available power for viscous dissipation 
        try:
            P_diss = param['P_diss_nu']
        except KeyError:
            try:
                P_diss = param['P_diss_nu']
            except KeyError:
                P_diss = P_diss
# determine time t_* that yields P_diss viscous dissipation
        t_star = np.sqrt(M_o*nu/(P_diss))
# get corresponding l_* lengthscale on the viscous line
        l_star = np.sqrt(t_star*nu)
# get l_perp of the VAC balance = intersection of Rossby line and viscous line
        [l_perp, t_nu_perp] = tl.function_intersect(R_o, t_Omega, -1, 1., 1./nu, 2)  
# -----
# plot arrow at viscous dissipation on the viscous line
        ax.annotate('',xy=(l_star, t_star), xytext=(l_star/6, t_star/2), arrowprops=dict(edgecolor=colors['nu'],facecolor=colors['nu']),color=colors['nu']) # P_diss arrow
# draw dissipation length scale l_star
        ax.plot([l_star, l_star], [ymin, ymax], dashes=(5,2,1,2,1,2), color = colors['nu'], label='$\ell_*$')
# draw force-balance length scale l_perp
        ax.plot([l_perp, l_perp], [ymin, ymax], dashes=(5,2,1,2,1,2,1,2), color = 'black', label='$\ell_\\perp$')
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# first point at (l_*,t_*) on the viscous line
# second point slope 2/3 (k-5/3 spectrum) to l_perp
        t_u_perp = t_star * (l_perp/l_star)**(2/3)
# third point slope 2/3 to intersection with Rossby line
        [l_u_Rossby, t_u_Rossby] = tl.function_intersect(R_o, t_Omega, -1, l_perp, t_u_perp, 2/3)  
# last point slope 0 to l=R_o 
        l_u_R_o = R_o
        t_u_R_o = t_u_Rossby
# create array of tau_u(ell) points
        l_u = np.array([l_star, l_perp,   l_u_Rossby, l_u_R_o])
        t_u = np.array([t_star, t_u_perp, t_u_Rossby, t_u_R_o])
# plot tau_u(ell) velocity line
        ax.plot(l_u, t_u, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at l_perp at mid-distance between t_u_perp and t_perp
        t_ff_perp = np.sqrt(t_u_perp*t_nu_perp)
# first point at intersection of thermal diffusion line and free-fall line (-1/4 tau-ell slope)
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_perp, t_ff_perp, -1/4, 1., 1./kappa, 2)  
# third point at intersection of free-fall line (-1/4 tau-ell slope) with l = R_o
        l_ff_R_o = R_o
        t_ff_R_o = (R_o/l_perp)**(-1/4)*t_ff_perp
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_perp,    l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_ff_perp, t_ff_R_o])
# plot tau_ff(ell) free-fall line
        ax.plot(l_ff, t_ff, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)
#
# --- construct and draw (tau=period,ell=wavelength) and free-fall point at convection threshold
# ----------------------------------------------------------------------------------------------
        if 0==0:
# following local marginal stability theory of Busse (1970). His equations (4.11) to (4.13).
# intersection  Rossby line with first encountered of t_nu(ell) or t_kappa(ell)
            [l_u_c, t_temp] = tl.function_intersect(R_o, 1./Omega, -1., 1., 1./max(nu,kappa), 2)
# l_u_c is the column width at threshold
# t_u_c is the drift period at threshold = t_kappa(lll_c)
            t_u_c = l_u_c**2/kappa
# t_rho_c at ell = R_o is the critical tau_rho (from critical Ra)
# at mid-distance between t_nu(lll_c) and t_kappa(lll_c)
            t_ff_c = np.sqrt(l_u_c**2/nu * l_u_c**2/kappa)
# plot rotating convection onset markers (thermal diffusivity)
            ax.plot(l_u_c,t_u_c, 'vk',markersize=12) # (tau_c,l_c) at convection threshold
            ax.plot([l_u_c, R_o],[t_ff_c, t_ff_c], ':', color=colors['rho']) # construction line
            ax.plot(R_o,t_ff_c, 'v', color=colors['rho'], markersize=12, clip_on=False) # (R_o, t_ff_c) tau_rho(R_o) at threshold
#
# =================================================== #
# MAC balance scenario (Magnetic-Archimedes-Coriolis) #
# =================================================== #
    elif force_balance == 'MAC':
# get available power for ohmic dissipation 
        try:
            P_diss = param['P_diss_eta']
        except KeyError:
            P_diss = P_diss
# determine time t_* that yields P_diss magnetic dissipation
        t_star = np.sqrt(eta*M_o/P_diss)
        l_diss = np.sqrt(eta*t_star)
# time tau_eta(R_o)
        t_eta_Ro = R_o**2/eta
# get l_* from MAC balance at l_* such that Rm(l_*) = 1
        l_star = R_o * (t_star/t_Omega * t_star/t_eta_Ro)**(1/2)
# deduce actual MAC balance ell as : l_perp = l_star
        l_perp = l_star
# -----
# plot arrow at Ohmic dissipation on magnetic diffusion line
        ax.annotate('',xy=(l_diss, t_star), xytext=(l_diss/6, t_star/2), arrowprops=dict(edgecolor=colors['eta'],facecolor=colors['eta']))
# draw P_diss attached to tau_eta line
        ax.plot([l_diss, l_star], [t_star, t_star], '--', color=colors['eta'])
# draw dissipation length scale l_star
        ax.plot([l_star, l_star], [ymin, ymax], dashes=(5,2,1,2,1,2), color = colors['b'], label='$\ell_*$')
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# second point at l_* on magnetic diffusion line (Rm(l_*)=1)
        t_u_eta = l_star**2/eta
        l_u_eta = l_star
# first point at the intersection of viscous line and k^-3 line (0 tau-ell slope)
        [l_u_nu, t_u_nu] = tl.function_intersect(l_u_eta, t_u_eta, 0, 1., 1./nu, 2)
# last point slope 1 to l=R_o 
        l_u_R_o = R_o
        t_u_R_o = t_u_eta*(R_o/l_u_eta)
# create array of tau_u(ell) points
        l_u = np.array([l_u_nu, l_u_eta, l_u_R_o])
        t_u = np.array([t_u_nu, t_u_eta, t_u_R_o])
# plot tau_u(ell) velocity line
        ax.plot(l_u, t_u, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_b(ell) magnetic line
# -----------------------------------------------
# second point at l_star and t_star
# first point at the intersection of magnetic diffusion line and k^-5 line (-1 tau-ell slope)
        [l_b_eta, t_b_eta] = tl.function_intersect(l_star, t_star, -1, 1., 1./eta, 2)
# third point slope 1 to l=R_o
        l_b_R_o = R_o
        t_b_R_o = t_star*(R_o/l_b_eta)
# create array of tau_b(ell) points
        l_b = np.array([l_b_eta, l_star, l_b_R_o])
        t_b = np.array([t_b_eta, t_star, t_b_R_o])
# plot tau_b(ell) magnetic line
        ax.plot(l_b, t_b, color=colors['b'], label='$b$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at l_star and t_star
# first point at intersection of thermal diffusion line and free-fall line (1/4 tau-ell slope)
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_star, t_star, 1/4, 1., 1./kappa, 2)  
# third point at intersection of free-fall line (1/4 tau-ell slope) with l = R_o
        l_ff_R_o = R_o
        t_ff_R_o = (R_o/l_ff_kappa)**(1/4)*t_ff_kappa
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_star, l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_star, t_ff_R_o])
# plot tau_ff(ell) free-fall line
        ax.plot(l_ff, t_ff, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)

# ======================================================================== #
# QG-MAC balance scenario (Quasi-Geostrophic Magnetic-Archimedes-Coriolis) #
# ======================================================================== #
    elif force_balance == 'QG-MAC':
# get available power for ohmic dissipation 
        try:
            P_diss = param['P_diss_eta']
        except KeyError:
            P_diss = P_diss
# determine time t_* that yields P_diss magnetic dissipation
        t_star = np.sqrt(eta*M_o/P_diss)
        l_diss = np.sqrt(eta*t_star)
# time tau_eta(R_o)
        t_eta_Ro = R_o**2/eta
# get l_* from QG-MAC balance at l_perp such that Rm(l_*) = 1
# --- with tau_u and tau_b propto ell^3/2 between l_* and l_perp
        l_star = R_o * ((l_perp/R_o)**(5/2) * t_star/t_Omega * t_star/t_eta_Ro)**(2/7)
# -----
# plot arrow at Ohmic dissipation on magnetic diffusion line
        ax.annotate('',xy=(l_diss, t_star), xytext=(l_diss/6, t_star/2), arrowprops=dict(edgecolor=colors['eta'],facecolor=colors['eta']))
# draw P_diss attached to tau_eta line
        ax.plot([l_diss, l_star], [t_star, t_star], '--', color=colors['eta'])
# draw dissipation length scale l_star
        ax.plot([l_star, l_star], [ymin, ymax], dashes=(5,2,1,2,1,2), color = colors['b'], label='$\ell_*$')
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# second point at l_* on magnetic diffusion line (Rm(l_*)=1)
        t_u_eta = l_star**2/eta
        l_u_eta = l_star
# first point at the intersection of viscous line and k^-3 line (0 tau-ell slope)
        [l_u_nu, t_u_nu] = tl.function_intersect(l_u_eta, t_u_eta, 0, 1., 1./nu, 2)
# last point slope 3/2 to l=R_o
        l_u_R_o = R_o
        t_u_R_o = t_u_eta*(R_o/l_u_eta)**(3/2)
# create array of tau_u(ell) points
        l_u = np.array([l_u_nu, l_u_eta, l_u_R_o])
        t_u = np.array([t_u_nu, t_u_eta, t_u_R_o])
# draw force-balance length scale l_perp
        ax.plot([l_perp, l_perp], [ymin, ymax], dashes=(5,2,1,2,1,2,1,2), color = 'black', label='$\ell_\\perp$')
# plot tau_u(ell) velocity line
        ax.plot(l_u, t_u, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_b(ell) magnetic line
# -----------------------------------------------
# second point at l_star and t_star
# first point at the intersection of magnetic diffusion line and k^-5 line (-1 tau-ell slope)
        [l_b_eta, t_b_eta] = tl.function_intersect(l_star, t_star, -1, 1., 1./eta, 2)
# third point slope 3/2 to l=R_o
        l_b_R_o = R_o
        t_b_R_o = t_star*(R_o/l_star)**(3/2)
# for later use : get t_b_perp (intersection of slope 3/2 with l_perp)
        t_b_perp = t_star*(l_perp/l_star)**(3/2)
# create array of tau_b(ell) points
        l_b = np.array([l_b_eta, l_star, l_b_R_o])
        t_b = np.array([t_b_eta, t_star, t_b_R_o])
# plot tau_b(ell) magnetic line
        ax.plot(l_b, t_b, color=colors['b'], label='$b$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at l_perp and t_b_perp
# first point at intersection of thermal diffusion line and free-fall line (1/4 tau-ell slope)
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_perp, t_b_perp, 1/4, 1., 1./kappa, 2)  
# third point at intersection of free-fall line (1/4 tau-ell slope) with l = R_o
        l_ff_R_o = R_o
        t_ff_R_o = (R_o/l_perp)**(1/4)*t_b_perp
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_perp,   l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_b_perp, t_ff_R_o])
# plot tau_ff(ell) free-fall line
        ax.plot(l_ff, t_ff, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)

# ================================================================================= #
# QG-MACa balance scenario (Quasi-Geostrophic Magnetic-Archimedes-Coriolis) variant #
# ================================================================================= #
# variant of QG-MAC with slope=1 tau_u(ell) and tau_b(ell) beyond l_star
    elif force_balance == 'QG-MACa':
# get available power for ohmic dissipation 
        try:
            P_diss = param['P_diss_eta']
        except KeyError:
            P_diss = P_diss
# determine time t_* that yields P_diss magnetic dissipation
        t_star = np.sqrt(eta*M_o/P_diss)
        l_diss = np.sqrt(eta*t_star)
# time tau_eta(R_o)
        t_eta_Ro = R_o**2/eta
# get l_* from QG-MAC balance at l_perp such that Rm(l_*) = 1
# --- with tau_u and tau_b propto ell^3/2 between l_* and l_perp
        l_star = R_o * ((l_perp/R_o)**(2) * t_star/t_Omega * t_star/t_eta_Ro)**(1/3)
# -----
# plot arrow at Ohmic dissipation on magnetic diffusion line
        ax.annotate('',xy=(l_diss, t_star), xytext=(l_diss/6, t_star/2), arrowprops=dict(edgecolor=colors['eta'],facecolor=colors['eta']))
# draw P_diss attached to tau_eta line
        ax.plot([l_diss, l_star], [t_star, t_star], '--', color=colors['eta'])
# draw dissipation length scale l_star
        ax.plot([l_star, l_star], [ymin, ymax], dashes=(5,2,1,2,1,2), color = colors['b'], label='$\ell_*$')
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# second point at l_* on magnetic diffusion line (Rm(l_*)=1)
        t_u_eta = l_star**2/eta
        l_u_eta = l_star
# first point at the intersection of viscous line and k^-3 line (0 tau-ell slope)
        [l_u_nu, t_u_nu] = tl.function_intersect(l_u_eta, t_u_eta, 0, 1., 1./nu, 2)
# last point slope 1 to l=R_o
        l_u_R_o = R_o
        t_u_R_o = t_u_eta*(R_o/l_u_eta)
# create array of tau_u(ell) points
        l_u = np.array([l_u_nu, l_u_eta, l_u_R_o])
        t_u = np.array([t_u_nu, t_u_eta, t_u_R_o])
# draw force-balance length scale l_perp
        ax.plot([l_perp, l_perp], [ymin, ymax], dashes=(5,2,1,2,1,2,1,2), color = 'black', label='$\ell_\\perp$')
# plot tau_u(ell) velocity line
        ax.plot(l_u, t_u, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_b(ell) magnetic line
# -----------------------------------------------
# second point at l_star and t_star
# first point at the intersection of magnetic diffusion line and k^-5 line (-1 tau-ell slope)
        [l_b_eta, t_b_eta] = tl.function_intersect(l_star, t_star, -1, 1., 1./eta, 2)
# third point slope 1 to l=R_o
        l_b_R_o = R_o
        t_b_R_o = t_star*(R_o/l_star)
# for later use : get t_b_perp (intersection of slope 1 with l_perp)
        t_b_perp = t_star*(l_perp/l_star)
# create array of tau_b(ell) points
        l_b = np.array([l_b_eta, l_star, l_b_R_o])
        t_b = np.array([t_b_eta, t_star, t_b_R_o])
# plot tau_b(ell) magnetic line
        ax.plot(l_b, t_b, color=colors['b'], label='$b$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at l_perp and t_b_perp
# first point at intersection of thermal diffusion line and free-fall line (1/4 tau-ell slope)
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_perp, t_b_perp, 1/4, 1., 1./kappa, 2)  
# third point at intersection of free-fall line (1/4 tau-ell slope) with l = R_o
        l_ff_R_o = R_o
        t_ff_R_o = (R_o/l_perp)**(1/4)*t_b_perp
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_perp,   l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_b_perp, t_ff_R_o])
# plot tau_ff(ell) free-fall line
        ax.plot(l_ff, t_ff, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)

# ======================================================================================= #
# QG-MAC_JA balance scenario (Quasi-Geostrophic Magnetic-Archimedes-Coriolis) a la Aubert #
# ======================================================================================= #
    elif force_balance == 'QG-MAC_JA':
# get available power for ohmic dissipation 
        try:
            P_diss = param['P_diss_eta']
        except KeyError:
            P_diss = P_diss
# determine time t_* that yields P_diss magnetic dissipation
        t_star = np.sqrt(eta*M_o/P_diss)
        l_diss = np.sqrt(eta*t_star)
# time tau_eta(R_o)
        t_eta_Ro = R_o**2/eta
# get l_* from QG-MAC balance at l_perp such that Rm(l_*) = 1
# --- with tau_u and tau_b propto ell^1/2 between l_* and l_perp and proto ell^3/2 from l_perp to R_o
        l_star = R_o * ((l_perp/R_o)**(3/2) * t_star/t_Omega * t_star/t_eta_Ro)**(2/5)
# -----
# plot arrow at Ohmic dissipation on magnetic diffusion line
        ax.annotate('',xy=(l_diss, t_star), xytext=(l_diss/6, t_star/2), arrowprops=dict(edgecolor=colors['eta'],facecolor=colors['eta']))
# draw P_diss attached to tau_eta line
        ax.plot([l_diss, l_star], [t_star, t_star], '--', color=colors['eta'])
# draw dissipation length scale l_star
        ax.plot([l_star, l_star], [ymin, ymax], dashes=(5,2,1,2,1,2), color = colors['b'], label='$\ell_*$')
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# second point at l_* on magnetic diffusion line (Rm(l_*)=1)
        t_u_eta = l_star**2/eta
        l_u_eta = l_star
# first point at the intersection of viscous line and k^-3 line (0 tau-ell slope)
        [l_u_nu, t_u_nu] = tl.function_intersect(l_u_eta, t_u_eta, 0, 1., 1./nu, 2)
# third point slope 1/2 to l=l_perp
        t_u_perp = t_u_eta*(l_perp/l_star)**(1/2)
# last point slope 3/2 to l=R_o
        l_u_R_o = R_o
        t_u_R_o = t_u_perp*(R_o/l_perp)**(3/2)
# create array of tau_u(ell) points
        l_u = np.array([l_u_nu, l_u_eta, l_perp,   l_u_R_o])
        t_u = np.array([t_u_nu, t_u_eta, t_u_perp, t_u_R_o])
# draw force-balance length scale l_perp
        ax.plot([l_perp, l_perp], [ymin, ymax], dashes=(5,2,1,2,1,2,1,2), color = 'black', label='$\ell_\\perp$')
# plot tau_u(ell) velocity line
        ax.plot(l_u, t_u, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_b(ell) magnetic line
# -----------------------------------------------
# second point at l_star and t_star
# first point at the intersection of magnetic diffusion line and k^-5 line (-1 tau-ell slope)
        [l_b_eta, t_b_eta] = tl.function_intersect(l_star, t_star, -1, 1., 1./eta, 2)
# third point slope 1/2 to l=l_perp
        t_b_perp = t_star*(l_perp/l_star)**(1/2)
# fourth point slope 3/2 to l=R_o
        l_b_R_o = R_o
        t_b_R_o = t_b_perp*(R_o/l_perp)**(3/2)
# create array of tau_b(ell) points
        l_b = np.array([l_b_eta, l_star, l_perp,   l_b_R_o])
        t_b = np.array([t_b_eta, t_star, t_b_perp, t_b_R_o])
# plot tau_b(ell) magnetic line
        ax.plot(l_b, t_b, color=colors['b'], label='$b$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at l_perp and t_b_perp
# first point at intersection of thermal diffusion line and free-fall line (1/4 tau-ell slope)
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_perp, t_b_perp, 1/4, 1., 1./kappa, 2)  
# third point at intersection of free-fall line (1/4 tau-ell slope) with l = R_o
        l_ff_R_o = R_o
        t_ff_R_o = (R_o/l_perp)**(1/4)*t_b_perp
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_perp,   l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_b_perp, t_ff_R_o])
# plot tau_ff(ell) free-fall line
        ax.plot(l_ff, t_ff, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)

# ============================================================ #
# IMAC balance scenario (Inertia-Magnetic-Archimedes-Coriolis) #
# ============================================================ #
    elif force_balance == 'IMAC':
# get available power for ohmic dissipation 
        try:
            P_diss_eta = param['P_diss_eta']
        except KeyError:
            P_diss_eta = P_diss/2
# get available power for viscous dissipation 
        try:
            P_diss_nu = param['P_diss_nu']
        except KeyError:
            P_diss_nu = P_diss/2
#
# --- construct and draw tau_u(ell) velocity line
# -----------------------------------------------
# first point at P_diss_nu on viscous line (231206)
        t_u_nu = np.sqrt(M_o*nu/(P_diss_nu))
        l_u_nu = np.sqrt(t_u_nu*nu)
# plot arrow at viscous  dissipation on viscous diffusion line
        ax.annotate('',xy=(l_u_nu, t_u_nu), xytext=(l_u_nu/6, t_u_nu/2), arrowprops=dict(edgecolor=colors['nu'],facecolor=colors['nu']))
# for later use : get l_Omega of Rossby=1 (intersection of K41 with t_Omega)
        [l_u_Omega, t_u_Omega] = tl.function_intersect(1, t_Omega, 0, l_u_nu, t_u_nu, 2/3)
# for later use : get l_u_eta of Rm=1 (intersection of K41 with t_Omega)
        [l_u_eta, t_u_eta] = tl.function_intersect(1, 1/eta, 2, l_u_nu, t_u_nu, 2/3)
# second point K41 at injection scale = scale height H or R_o-R_i following K41 (2/3 slope)
        l_u_H = min(HH,R_o-R_i)
        t_u_H = t_u_nu*(l_u_H/l_u_nu)**(2/3)
# third point at intersection of slope 2/3 (K41) with Rossby line inside tangent cylinder 231206
        [l_u_Rossby, t_u_Rossby] = tl.function_intersect(R_i, t_Omega*(R_o-R_i)/R_i, -1., l_u_H, t_u_H, 2/3)
# fourth point at intersection with (r=R_o) with slope 2 (non-azimutal velocity) 231206
        l_u_R_o = R_o
        t_u_R_o = t_u_H*(R_o/l_u_H)**2
# adjustment if injection scale larger than l_Omega
        if l_u_H > l_u_Omega:
            l_u_H = l_u_Omega
            t_u_H = t_u_Omega
# create array of tau_u(ell) points
        l_u = np.array([l_u_nu, l_u_H, l_u_Omega, l_u_Rossby, l_u_R_o])
        t_u = np.array([t_u_nu, t_u_H, t_u_Omega, t_u_Rossby, t_u_R_o])
# draw force-balance length scale l_omega
        ax.plot([l_u_Omega, l_u_Omega], [ymin, ymax], dashes=(5,2,1,2,1,2,1,2), color=colors['day'], label='$\ell_\\Omega$')
# plot tau_u(ell) velocity line
        ax.plot(l_u, t_u, color=colors['u'], label='$u$', linewidth=4, alpha=0.5)

# --- construct and draw tau_b(ell) magnetic line
# -----------------------------------------------
# first point at P_diss_eta on magnetic line (231206)
        t_b_eta = np.sqrt(M_o*eta/(P_diss_eta))
        l_b_eta = np.sqrt(t_b_eta*eta)
# plot arrow at Ohmic dissipation on magnetic diffusion line
        ax.annotate('',xy=(l_b_eta, t_b_eta), xytext=(l_b_eta/6, t_b_eta/2), arrowprops=dict(edgecolor=colors['eta'],facecolor=colors['eta']))
# second point at (l_u_Omega,t_Omega) for MAC + CIA balance
        l_b_Omega = l_u_Omega
        t_b_Omega = t_Omega
# third point at intersection with r=R_o with slope 2
        l_b_R_o = R_o
        t_b_R_o = t_Omega*(R_o/l_u_Omega)**2
# create array of tau_b(ell) points
        l_b = np.array([l_b_eta, l_b_Omega, l_b_R_o])
        t_b = np.array([t_b_eta, t_b_Omega, t_b_R_o])
# plot tau_b(ell) magnetic line
        ax.plot(l_b, t_b, color=colors['b'], label='$b$', linewidth=4, alpha=0.5)
#
# --- construct and draw tau_ff(ell) free-fall line
# -------------------------------------------------
# second point at Ro=1 (intersection of line t_u with t_Omega)
        l_ff_Omega = l_u_Omega
        t_ff_Omega = t_Omega
# first point at intersection of kappa line with slope 1/2 tau_ff line
        [l_ff_kappa, t_ff_kappa] = tl.function_intersect(l_ff_Omega, t_ff_Omega, 1./2, 1., 1./kappa, 2)
# third point at intersection of line with slope 1/2 tau_ff with ell=R_o-R_i
        l_ff_R_o = R_o-R_i
        t_ff_R_o = t_ff_kappa * ((R_o-R_i)/l_ff_kappa)**0.5
# create array of tau_ff(ell) points
        l_ff = np.array([l_ff_kappa, l_ff_Omega, l_ff_R_o])
        t_ff = np.array([t_ff_kappa, t_ff_Omega, t_ff_R_o])
# plot tau_ff(ell) free-fall line
        ax.plot(l_ff, t_ff, color=colors['rho'], label='$\\rho$', linewidth=4, alpha=0.5)


# o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o-o #

# =========================================================================== #
#                                                                             #
#         label dimensionless numbers at intersections if requested           #
#                                                                             #
# =========================================================================== #
    if plot_dimensionless:
# Reynolds number
        if 'l_u_nu' in locals():
            plt.text(l_u_nu,t_u_nu,"$\\bigcirc$", size=20, ha='center', va='center') # Re(l)~1
            plt.text(l_u_nu/shift,t_u_nu*shift,"$\\mathrm{Re}$", size=18, ha='right', va='bottom') 
# magnetic Reynolds number
        if 'l_u_eta' in locals() and with_mag:
            plt.text(l_u_eta,t_u_eta,"$\\bigcirc$", size=20, ha='center', va='center') # Rm(l)~1
            plt.text(l_u_eta/shift,t_u_eta*shift,"$\\mathrm{Rm}$", size=18, ha='right', va='bottom') 
# Rossby number
        if 'l_u_Omega' in locals():
            plt.text(l_u_Omega,t_Omega,"$\\bigcirc$", size=20, ha='center', va='center') # Ro(l)~1
            plt.text(l_u_Omega/shift,t_Omega*shift,"$\\mathrm{Ro}$", size=18, ha='right', va='bottom') 

# *******************
    ax.legend(loc='upper left')
    ax.grid(visible=None)
    return
# ------------------------------------------------------------------------------