#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
====================================
plot tau-ell regime diagram template
====================================
Small routine to plot a tau-ell regime diagram template for rotating convective dynamo
in a spherical shell.
From 'Dynamic regimes in planetary cores: tau-ell diagrams', Nataf H-C. and N. Schaeffer, Compte-rendu Geosciences (2024).

This function returns: fig, ax, colors

240206 HCN : include plot of P_diss arrows
240205 HCN : small modifs to add scaling laws and convection onset
240130 HCN : small adjustments for full sphere (not completed)
231214 HCN : created from plot_rotating_shell.py *** pb position QG power markers ***
"""

from read_parameters import read_parameters
import tau_ell_lib as tl
import numpy as np
import matplotlib.pyplot as plt

# ===========================================================================
def plot_template(object, template, xmin, xmax, ymin, ymax, y_sec, with_mag, stress_free, shell_case, plot_dimensionless, plot_scaling_laws, plot_convection_onset):
# ===========================================================================
#
    title = object + ' regime diagram template'
#
# --> get needed object's parameters
# ----------------------------------
    param = read_parameters(object)
    R_o = param['R_o']
    R_i = param['R_i']
    Omega = param['Omega']
    M_o = param['M_o']
    nu = param['nu']
    kappa = param['kappa']
    kappa_chi = param['kappa_chi']
    eta = param['eta']
    P_diss = param['P_diss']
    gravity = param['gravity']
    alpha = param['alpha']
    C_p = param['C_p']
    rho = param['rho']
    t_SV = param['t_SV']
    t_A0 = param['t_A0']
    obs_list = param['obs_list']
    object_type = param['object_type']
# -------------------------------------------------------
# a few auxiliary parameters
    R_av = (R_o+R_i)/2 # [m] shell mean radius
    V_shell = 4*np.pi/3 * (R_o**3 - R_i**3) # [m^3] volume of the complete shell
    HH = C_p/gravity/alpha # [m] scale height
    if shell_case:
        DD = R_o-R_i # [m] shell thickness
    else:
        DD = R_o
##
#if object_type == 'star_sim': # non-compressible simulations
#    HH = dd
    t_Omega = 1./Omega # [s] Omega^(-1) reference time
    l_Ekman = np.sqrt(nu/Omega) # [m] Ekman layer thickness
    Ekman = nu/Omega/DD**2 # Ekman number
    t_spinup = DD/np.sqrt(nu*Omega) # [s] spinup time
    l_spinup = DD * Ekman**(1./4.) # [m] length for transition to spinup
#
# launch the figure
# ---------------------------------------------------------------------
    fig, ax, colors = tl.tau_ell_setup(title,xmin,xmax,ymin,ymax,y_sec)
# ---------------------------------------------------------------------
# a few graphical stuff 
    shift = 1.3  # shift for labels
    t_symb = ymax*0.1
    marker_list = ["o","x","s","+"]
    if object_type in {'star','star_sim'}:
        power_label_list = [12, 15, 18, 21, 24]
    elif object_type == 'planetary_core':
        power_label_list = [3, 6, 9, 12]
    elif object_type == 'planetary_core_sim':
        power_label_list = [12, 15, 18]
    elif object_type == 'experiment':
##        power_label_list = [-3, 0, 3, 6]
        power_label_list = [-3, 3, 6]
    else:
        power_label_list = [3, 6, 9, 12]
#
# construct length vector
    pmin = np.floor(np.log10(xmin))
    pmax = np.ceil(np.log10(xmax))
    length = 10**np.arange(pmin,pmax,1.0) # [m] length vector
    length[-1] = xmax
#
# =========================================================================== #
#                                                                             #
#                 construct and draw lines from parameters                    #
#                                                                             #
# =========================================================================== #
#
# ---- plot D and R_av (or R_o) and H
    if shell_case:
        ax.plot([DD, DD], [ymin, ymax], '-k')
        plt.text(DD, ymax*1.5, "$D$", ha='center', va='bottom')
        plt.text(R_av, ymin*0.6, r"$\bar R$", ha='center', va='top')
    else:
        plt.text(R_o, ymin*0.6, "$R_o$", ha='center', va='top')
#    
    if HH<xmax:
        ax.plot(HH, ymin, '^k', markersize=10,clip_on=False)
        plt.text(HH, ymin*1.5, "$H$", ha='center', va='bottom')
# ---- object's symbol
    try:
        object_symbol = param['object_symbol']
        plt.text(xmax/600,ymin*40,object_symbol,size=50)
    except KeyError:
        print('...')
#
# ---- draw day line
    ax.plot([xmin, xmax], [t_Omega, t_Omega], '-.', color=colors['day'], label='$t_\\Omega$')
##    plt.text(xmin/shift, t_Omega, "$t_\Omega$", va='center', ha='right', color=colors['day'])
    plt.text(xmax*shift, t_Omega, "$t_\Omega$", va='center', ha='left', color=colors['day'])
#
# --- construct and draw thermal diffusion line
    t_kappa = length**2/kappa # [s]
    ax.plot(length, t_kappa, color=colors['kappa'])
    plt.text(np.sqrt(kappa*t_symb*0.5)*0.8, t_symb*0.5, "$\\kappa$", ha = 'right', va='bottom', color=colors['kappa'])
#
# --- construct and draw chemical diffusion line
    t_kappa_chi = length**2/kappa_chi ; # [s]
    ax.plot(length, t_kappa_chi, color=colors['chi'])
    plt.text(np.sqrt(kappa_chi*t_symb)*0.8, t_symb, "$\\chi$", ha = 'right', va='bottom', color=colors['chi'])
#
# ---------------------------------------------------
# viscous and rotation ingredients   
# ---------------------------------------------------
# --- construct and draw viscous diffusion line
    length_nu = length.copy() # viscosity length will be trimed above spinup transition
    t_nu = length_nu**2/nu # [s]
# to suppress part of viscous line above t_spinup
    for ii in range(len(length_nu)):
        t_nu[ii] = min(t_nu[ii],t_spinup)
        length_nu[ii] = min(length_nu[ii],l_spinup)
    ax.plot(length_nu, t_nu, color=colors['nu']) # viscous diffusion line
    ax.plot([l_spinup, R_o], [t_spinup, t_spinup], color=colors['nu']) # spinup line
    plt.text(np.sqrt(nu*t_symb*0.2)*0.8, t_symb*0.2, "$\\nu$", ha = 'right', va='bottom', color=colors['nu'])
# construct and draw viscous dissipated power markers
# ----------------------------------------------------
    (l_nu_power, t_nu_power, index, label) = tl.construct_power_markers(nu, M_o)
# delete power markers that fall beyond the spinup time (very pythonesq syntax!)
    l_nu_power_plot, t_nu_power_plot = [], []
    t_nu_power_plot[:] = [x for x in t_nu_power if x<t_spinup]
    l_nu_power_plot[:] = [x for x in l_nu_power if x<np.sqrt(t_spinup*nu)]
# plot bulk viscous dissipation markers
    ax.plot(l_nu_power_plot, t_nu_power_plot, 's', color=colors['nu'], fillstyle='none', markersize=12)
# label selected dissipation markers
#    for exp in (12, 15, 18, 21, 24):
    for exp in power_label_list:
        ind = index[exp]
        l_exp = l_nu_power[ind]
        t_exp = t_nu_power[ind]
        if exp == 3:
            ax.plot(l_exp, t_exp, 's', color=colors['nu'], markersize=12, alpha=0.2)
        if exp == 12:
            ax.plot(l_exp, t_exp, 's', color=colors['nu'], markersize=12)
        if (l_exp>xmin and l_exp<xmax and t_exp>ymin and t_exp<ymax):
            plt.text(l_exp*0.6, t_exp, label[exp], ha = 'right', va='bottom', color=colors['nu'], size=max(9,exp*0.7))
#
# construct and draw Rossby lines
# ----------------------------------------------------
# - - - - inside tangent cylinder - - - -
# --> Rossby line built on length-scale D = R_o-Ri
    l_Rossby_in, t_Rossby_in = [], [] # create lists
    [lll, ttt] = tl.function_intersect(R_o-R_i, 1./Omega, -1., 1., 1./nu, 2) # intersection  Rossby time with avec t_nu
    l_Rossby_in.append(lll) # first point
    t_Rossby_in.append(ttt)
    l_Rossby_in.append(R_o-R_i) # second point
    t_Rossby_in.append(t_Omega)
#
# - - - - outside tangent cylinder - - - -
# --> Rossby line built on length-scale R_o
    l_Rossby_out, t_Rossby_out = [], [] # create lists
    [lll, ttt] = tl.function_intersect(R_o, 1./Omega, -1., 1., 1./nu, 2) # intersection  Rossby time with avec t_nu
    l_Rossby_out.append(lll) # first point
    t_Rossby_out.append(ttt)
    l_Rossby_out.append(R_o) # second point
    t_Rossby_out.append(t_Omega)
#
# depending on whether the largest volume is inside or outside the tangent cylinder (TC),
# we select Rossby waves within or outside the TC
    if shell_case: # just so for the Sun...
        l_Rossby = l_Rossby_in
        t_Rossby = t_Rossby_in
    else:
        l_Rossby = l_Rossby_out
        t_Rossby = t_Rossby_out
# plot Rossby line and construct QG power markers
    line_kwargs = {'color':colors['Rossby'], 'alpha':0.5}
    tl.plot_wave(l_Rossby, t_Rossby, npts=100, nper=20, famp=0.2, line_kwargs=line_kwargs)
    (l_nu_QG_power, t_nu_QG_power, index, label) = tl.construct_QG_power_markers(l_Ekman, Omega, DD, M_o)
#
# construct and draw QG viscous dissipated power markers
# ----------------------------------------------------
    if not stress_free: # QG dissipation only for no-slip
#        (l_nu_QG_power, t_nu_QG_power, index, label) = tl.construct_QG_power_markers(l_Ekman, Omega, R_o, M_o)
# delete QG power markers that fall beyond the viscous line
        l_nu_power_plot, t_nu_power_plot = [], []
        for ii in range(len(l_nu_QG_power)):
            if t_nu_QG_power[ii]<t_Rossby[0] and l_nu_QG_power[ii]>l_Rossby[0]:
                t_nu_power_plot.append(t_nu_QG_power[ii])
                l_nu_power_plot.append(l_nu_QG_power[ii])
# plot Ekman layer viscous dissipation markers
        ax.plot(l_nu_power_plot, t_nu_power_plot, 'd', color=colors['nu'], fillstyle='none', markersize=12)
# label selected QG dissipation markers
        for exp in power_label_list:
            ind = index[exp]
            l_exp = l_nu_QG_power[ind]
            t_exp = t_nu_QG_power[ind]
            if exp == 3:
                ax.plot(l_exp, t_exp, 'd', color=colors['nu'], markersize=12, alpha=0.2)
            if exp == 12:
                ax.plot(l_exp, t_exp, 'd', color=colors['nu'], markersize=12)
            if (l_exp>xmin and l_exp<xmax and t_exp>ymin and t_exp<ymax):
                plt.text(l_exp*0.7, t_exp, label[exp], ha = 'right', va='bottom', color=colors['nu'], size=max(9,exp))
#
# ---------------------------------------------------
# magnetic ingredients   
# ---------------------------------------------------
    if with_mag:
    
# --- construct and draw magnetic diffusion line
        t_eta = length**2/eta # [s]
        ax.plot(length, t_eta, color=colors['eta'])
        plt.text(np.sqrt(eta*t_symb*0.4)*0.8, t_symb*0.4, "$\\eta$", ha = 'right', va='bottom', color=colors['eta'])
# construct and draw magnetic dissipated power markers
# ----------------------------------------------------
        (l_eta_power, t_eta_power, index, label) = tl.construct_power_markers(eta, M_o)
# plot magnetic dissipation markers
        ax.plot(l_eta_power, t_eta_power, 's', color=colors['eta'], fillstyle='none', markersize=12)
# label selected dissipation markers
        for exp in power_label_list:
            ind = index[exp]
            l_exp = l_eta_power[ind]
            t_exp = t_eta_power[ind]
            if exp == 3:
                ax.plot(l_exp, t_exp, 's', color=colors['eta'], markersize=12, alpha=0.2)
            if exp == 12:
                ax.plot(l_exp, t_exp, 's', color=colors['eta'], markersize=12)
#            if (l_exp>xmin and l_exp<xmax):
            if (l_exp>xmin and l_exp<xmax and t_exp>ymin and t_exp<ymax):
                plt.text(l_exp*0.6, t_exp, label[exp], ha = 'right', va='bottom', color=colors['eta'], size=max(9,exp*0.7))
#
# construct and draw Alfven wave line (slope 1 anchored on (DD, t_b_obs[0])
# ---------------------------------------------------------------------------
        t_AAA = t_A0
# first point at intersection with magnetic diffusion line
        [l_Alfven_eta, t_Alfven_eta] = tl.function_intersect(DD, t_AAA, 1, 1., 1./eta, 2)
# second point at (DD,t_AAA)
# create array of tau_Alfven(ell) points
        l_Alfven = np.array([l_Alfven_eta, DD])
        t_Alfven = np.array([t_Alfven_eta, t_AAA])
# draw Alfven line
        line_kwargs = {'color':colors['Alfven'], 'alpha':0.5}
        tl.plot_wave(l_Alfven, t_Alfven, npts=150, nper=30, famp=0.2, line_kwargs=line_kwargs)
#
# =========================================================================== #
#                                                                             #
#                      plot P_diss dissipation arrows                         #
#                                                                             #
# =========================================================================== #
# if repartition known (for DNS principally), use it
# - - - - - - - - - - - - - - - - - - - - - - - - - -
# --- QG dissipation in Ekman layers, if available
    try:
        P_diss_nu_QG = param['P_diss_nu_QG']
        flag_nu_QG_diss = True
# determine time that yields P_diss QG viscous dissipation
        t_diss_nu_QG = (M_o*DD*l_Ekman/Omega/P_diss_nu_QG)**(1./4.)
# get corresponding lengthscale on the Rossby line
        l_diss_nu_QG = DD/Omega/t_diss_nu_QG # longueur sur ligne tau_Rossby correspondante
        ax.annotate('',xy=(l_diss_nu_QG, t_diss_nu_QG), xytext=(l_diss_nu_QG/3, t_diss_nu_QG/1.5), annotation_clip=False, arrowprops=dict(edgecolor=colors['nu'], facecolor=colors['nu']), color=colors['nu']) # P_diss_nu_QG arrow
    except KeyError:
        flag_nu_QG_diss = False
#
# --- bulk viscous dissipation, if available   
    try:
        P_diss_nu = param['P_diss_nu']
        flag_nu_diss = True
# determine time that yields P_diss bulk viscous dissipation
        t_diss_nu = np.sqrt(nu*M_o/P_diss_nu)
# get corresponding lengthscale on the viscous line
        l_diss_nu = np.sqrt(nu*t_diss_nu)
        ax.annotate('',xy=(l_diss_nu, t_diss_nu), xytext=(l_diss_nu/3, t_diss_nu/1.5), arrowprops=dict(edgecolor=colors['nu'],facecolor=colors['nu']),color=colors['nu']) # P_diss_nu arrow
    except KeyError:
        flag_nu_diss = False
#
# --- Ohmic dissipation, if available
    try:
        P_diss_eta = param['P_diss_eta']
        flag_eta_diss = True
# determine time that yields P_diss bulk Ohmic dissipation
        t_diss_eta = np.sqrt(eta*M_o/P_diss_eta)
# get corresponding lengthscale on the magnetic diffusion line
        l_diss_eta = np.sqrt(eta*t_diss_eta)
        ax.annotate('',xy=(l_diss_eta, t_diss_eta), xytext=(l_diss_eta/3, t_diss_eta/1.5), arrowprops=dict(edgecolor=colors['eta'],facecolor=colors['eta'])) # P_diss_eta arrow
    except KeyError:
        flag_eta_diss = False
#
# --> Ohmic to viscous dissipation ratio (used in Christensen scaling law)
    if with_mag:
        if flag_nu_diss and flag_eta_diss:
            f_Ohm = P_diss_eta / (P_diss_eta + P_diss_nu)
        else:
            f_Ohm = 1
        print('f_Ohm = ' + str(f_Ohm))

# if repartition unknown plot P_diss only for template (P_diss will be distributed by scenario) 
# - - - - - - - - - - - - - - - - - - - - - - - - - - -
    if (template and (not flag_nu_QG_diss) and (not flag_nu_diss) and (not flag_eta_diss)):
# ---- plot P_diss arrow on the viscous line if template only and stress-free
        if stress_free:
            t_diss = np.sqrt(M_o*nu/P_diss)
            l_diss = np.sqrt(t_diss*nu)
            ax.annotate('',xy=(l_diss, t_diss), xytext=(l_diss/6, t_diss/2), arrowprops=dict(edgecolor=colors['nu'],facecolor=colors['nu']),color=colors['nu']) # P_diss arrow
#
# ---- plot P_diss arrow on Rossby line if template only and no slip
        else:
            t_star = (M_o*DD*l_Ekman/Omega/P_diss)**(1./4.)
            l_star = DD/Omega/t_star
            ax.annotate('',xy=(l_star, t_star), xytext=(l_star/6, t_star/2), annotation_clip=False, arrowprops=dict(edgecolor=colors['nu'],facecolor=colors['nu']),color=colors['nu']) # P_diss arrow
#
# =========================================================================== #
#                                                                             #
#                   plot observational data if available                      #
#                                                                             #
# =========================================================================== #
# plot all tau_obs(ell) if available
    try:
        ii=0
        for dico in obs_list:
            type = dico['type']
            ell_array = dico['ell']
            tau_array = dico['tau']
            marker = marker_list[0] # 'o' marker and followers
# ---- velocity u -----
            if type in {'u'}:
                marker = marker_list[ii]    
                for nn in range(len(ell_array)):
                    ax.plot(ell_array[nn], tau_array[nn], marker=marker, color=colors['u'], markersize=max(10-nn,1), clip_on=False)
                if ii==0:
                    plt.text(R_o*shift, tau_array[0], "$t_{u}^{obs}$", ha='left', va='center', color=colors['u'])
##                    plt.text(R_o*shift, tau_array[0], "$T_{U}$", ha='left', va='center', color=colors['u'])
                ii=ii+1
# ---- velocity u zonal -----
            elif type in {'u_zonal'}:       
                for nn in range(len(ell_array)):
                    ax.plot(ell_array[nn], tau_array[nn], marker=marker, color=colors['u'], alpha=0.2, markersize=max(10-nn,1), clip_on=False)
                plt.text(R_o*shift, tau_array[0], "$t_{u_{zonal}}^{obs}$", ha='left', va='center', color=colors['u'], alpha=0.4)
# ---- density rho -----
            elif type in {'rho'}:       
                for nn in range(len(ell_array)):
                    ax.plot(ell_array[nn], tau_array[nn], marker=marker, color=colors['rho'], markersize=max(10-nn,1), clip_on=False)
                plt.text(R_o*shift, tau_array[0], "$t_{\\rho}^{obs}$", ha='left', va='center', color=colors['rho'])
# ---- magnetic field b -----
            elif type in {'b'}:
                if with_mag:      
                    for nn in range(len(ell_array)):
                        ax.plot(ell_array[nn], tau_array[nn], marker=marker, color=colors['b'], markersize=max(10-nn,1), clip_on=False)
                    plt.text(R_o*shift, tau_array[0], "$t_{b}^{obs}$", ha='left', va='center', color=colors['b'])
##                    plt.text(R_o*shift, tau_array[0], "$T_{B}$", ha='left', va='center', color=colors['b'])
#           
    except KeyError:
        print('no observed tau(ell)')
#
# =========================================================================== #
#                                                                             #
#                      plot scaling laws if requested                         #
#                                                                             #
# =========================================================================== #
# dynamo scaling laws for Earth's core geometry from Christensen (2010) and Davidson (2013)
    if (with_mag and plot_scaling_laws):
        (OK, tau_u_Uli) = tl.scaling_law('Christensen_2010','u',R_o,M_o,P_diss,Omega,rho,f_Ohm)
        if OK:
            ax.plot(R_o, tau_u_Uli, '*', color=colors['u'], markersize=10, clip_on=False)
        (OK, tau_b_Uli) = tl.scaling_law('Christensen_2010','b',R_o,M_o,P_diss,Omega,rho,f_Ohm)
        if OK:
            ax.plot(R_o, tau_b_Uli, '*', color=colors['b'], markersize=10, clip_on=False)
        (OK, tau_u_Davidson) = tl.scaling_law('Davidson_2013','u',R_o,M_o,P_diss,Omega,rho,f_Ohm)
        if OK:
            ax.plot(R_o, tau_u_Davidson, 'x', color=colors['u'], markersize=10, clip_on=False)
#
# =========================================================================== #
#                                                                             #
#                plot convection onset values if requested                    #
#                                                                             #
# =========================================================================== #
# non-magnetic rotating convection onset ell_c, tau_{u_c}, tau_{rho_c}
# following local marginal stability theory of Busse (1970). His equations (4.11) to (4.13).
# see Appendix B of Nataf & Schaeffer (2024)
# ***** to be done for rotating convection in a spherical shell ******
    if plot_convection_onset:
# --- construct and draw (tau=period,ell=wavelength) and free-fall point at convection threshold
# intersection  Rossby line with first encountered of t_nu(ell) or t_kappa(ell)
        [lll_c, t_temp] = tl.function_intersect(R_o, 1./Omega, -1., 1., 1./max(nu,kappa), 2)
# lll_c is the column width at threshold
# ttt_c is the drift period at threshold = t_kappa(lll_c)
        ttt_c = lll_c**2/kappa
# t_rho_c at ell = R_o is the critical tau_rho (from critical Ra)
# at mid-distance between t_nu(lll_c) and t_kappa(lll_c)
        t_ff_c = np.sqrt(lll_c**2/nu * lll_c**2/kappa)
# plot rotating convection onset markers (thermal diffusivity)
        ax.plot(lll_c,ttt_c, 'vk',markersize=12) # (tau_c,l_c) at convection threshold
        ax.plot([lll_c, R_o],[t_ff_c, t_ff_c], ':', color=colors['rho'])
        ax.plot(R_o,t_ff_c, 'v', color=colors['rho'], markersize=12, clip_on=False) # (R_o, t_ff_c) tau_rho(R_o) at threshold
#
# =========================================================================== #
#                                                                             #
#         label dimensionless numbers at intersections if requested           #
#                                                                             #
# =========================================================================== #
    if plot_dimensionless:
# plot intersections and label with ell-scale dimensionless numbers
# --- Ekman number
        [lll, ttt] = tl.function_intersect(DD, t_Omega, 0, 1., 1./nu, 2)
        plt.text(lll,ttt,"$\\bigcirc$", size=20, ha='center', va='center') # E(l)~1
        plt.text(lll/shift,ttt*shift,"$\\mathrm{Ek}$", size=18, ha='right', va='bottom')
# --- thermal Ekman number
        [lll, ttt] = tl.function_intersect(DD, t_Omega, 0, 1., 1./kappa, 2)
        plt.text(lll,ttt,"$\\bigcirc$", size=20, ha='center', va='center') # E_kappa(l)~1
        plt.text(lll*shift,ttt/shift,"$\\mathrm{Ek}_\\kappa$", size=18, ha='left', va='top')
        if with_mag:
# --- magnetic Ekman number
            [lll, ttt] = tl.function_intersect(DD, t_Omega, 0, 1., 1./eta, 2)
            plt.text(lll,ttt,"$\\bigcirc$", size=20, ha='center', va='center') # E_eta(l)~1
            plt.text(lll*shift,ttt/shift,"$\\mathrm{Ek}_\\eta$", size=18, ha='left', va='top')
# --- Lundquist number
            [l_Lu, t_Lu] = tl.function_intersect(DD, t_AAA, 1., 1, 1./eta, 2)
            plt.text(l_Lu,t_Lu,"$\\bigcirc$", size=20, ha='center', va='center') # \Lu(l)~1
            plt.text(l_Lu/shift,t_Lu*shift,"$\\mathrm{Lu}$", size=18, ha='right', va='bottom')
# --- Lehnert number
            [l_Le, t_Le] = tl.function_intersect(DD, t_AAA, 1., DD, t_Omega, 0)
            plt.text(l_Le,t_Le,"$\\bigcirc$", size=20, ha='center', va='center') # \lambda(l)~1
            plt.text(l_Le*shift,t_Le/shift,"$\\lambda$", size=18, ha='left', va='top')
#
    ax.grid()
    ax.legend(loc='upper left')
    return (fig, ax, colors)